#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# Create a window for cropping seismic survey


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.basic.data import data as basic_data
from cognitivegeo.src.basic.matdict import matdict as basic_mdt
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class cropsurvey(object):

    survinfo = {}
    seisdata = {}
    #
    iconpath = os.path.dirname(__file__)
    dialog = None

    def setupGUI(self, CropSurvey):
        CropSurvey.setObjectName("CropSurvey")
        CropSurvey.setFixedSize(480, 280)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/survey.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        CropSurvey.setWindowIcon(icon)
        self.lblsrvinfo = QtWidgets.QLabel(CropSurvey)
        self.lblsrvinfo.setObjectName("lblsrvinfo")
        self.lblsrvinfo.setGeometry(QtCore.QRect(10, 10, 150, 30))
        self.lblstart = QtWidgets.QLabel(CropSurvey)
        self.lblstart.setObjectName("lblstart")
        self.lblstart.setGeometry(QtCore.QRect(120, 50, 80, 30))
        self.lblend = QtWidgets.QLabel(CropSurvey)
        self.lblend.setObjectName("lblend")
        self.lblend.setGeometry(QtCore.QRect(220, 50, 80, 30))
        self.lblstep = QtWidgets.QLabel(CropSurvey)
        self.lblstep.setObjectName("lblstep")
        self.lblstep.setGeometry(QtCore.QRect(320, 50, 40, 30))
        self.lblinl = QtWidgets.QLabel(CropSurvey)
        self.lblinl.setObjectName("lblinl")
        self.lblinl.setGeometry(QtCore.QRect(10, 90, 100, 30))
        self.lblxl = QtWidgets.QLabel(CropSurvey)
        self.lblxl.setObjectName("lblxl")
        self.lblxl.setGeometry(QtCore.QRect(10, 130, 100, 30))
        self.lblz = QtWidgets.QLabel(CropSurvey)
        self.lblz.setObjectName("lblz")
        self.lblz.setGeometry(QtCore.QRect(10, 170, 100, 30))
        self.ldtinlstart = QtWidgets.QLineEdit(CropSurvey)
        self.ldtinlstart.setObjectName("ldtinlstart")
        self.ldtinlstart.setGeometry(QtCore.QRect(120, 90, 80, 30))
        self.ldtinlend = QtWidgets.QLineEdit(CropSurvey)
        self.ldtinlend.setObjectName("ldtinlend")
        self.ldtinlend.setGeometry(QtCore.QRect(220, 90, 80, 30))
        self.ldtinlstep = QtWidgets.QLineEdit(CropSurvey)
        self.ldtinlstep.setObjectName("ldtinlstep")
        self.ldtinlstep.setGeometry(QtCore.QRect(320, 90, 40, 30))
        self.lblinlitvl = QtWidgets.QLabel(CropSurvey)
        self.lblinlitvl.setObjectName("lblinlitvl")
        self.lblinlitvl.setGeometry(QtCore.QRect(370, 90, 10, 30))
        self.cbbinlitvl = QtWidgets.QComboBox(CropSurvey)
        self.cbbinlitvl.setObjectName("cbbinlitvl")
        self.cbbinlitvl.setGeometry(QtCore.QRect(385, 90, 40, 30))
        self.ldtxlstart = QtWidgets.QLineEdit(CropSurvey)
        self.ldtxlstart.setObjectName("ldtxlstart")
        self.ldtxlstart.setGeometry(QtCore.QRect(120, 130, 80, 30))
        self.ldtxlend = QtWidgets.QLineEdit(CropSurvey)
        self.ldtxlend.setObjectName("ldtxlend")
        self.ldtxlend.setGeometry(QtCore.QRect(220, 130, 80, 30))
        self.ldtxlstep = QtWidgets.QLineEdit(CropSurvey)
        self.ldtxlstep.setObjectName("ldtxlstep")
        self.ldtxlstep.setGeometry(QtCore.QRect(320, 130, 40, 30))
        self.lblxlitvl = QtWidgets.QLabel(CropSurvey)
        self.lblxlitvl.setObjectName("lblxlitvl")
        self.lblxlitvl.setGeometry(QtCore.QRect(370, 130, 10, 30))
        self.cbbxlitvl = QtWidgets.QComboBox(CropSurvey)
        self.cbbxlitvl.setObjectName("cbbxlitvl")
        self.cbbxlitvl.setGeometry(QtCore.QRect(385, 130, 40, 30))
        self.ldtzstart = QtWidgets.QLineEdit(CropSurvey)
        self.ldtzstart.setObjectName("ldtzstart")
        self.ldtzstart.setGeometry(QtCore.QRect(120, 170, 80, 30))
        self.ldtzend = QtWidgets.QLineEdit(CropSurvey)
        self.ldtzend.setObjectName("ldtzend")
        self.ldtzend.setGeometry(QtCore.QRect(220, 170, 80, 30))
        self.ldtzstep = QtWidgets.QLineEdit(CropSurvey)
        self.ldtzstep.setObjectName("ldtzlstep")
        self.ldtzstep.setGeometry(QtCore.QRect(320, 170, 40, 30))
        self.lblzitvl = QtWidgets.QLabel(CropSurvey)
        self.lblzitvl.setObjectName("lblzitvl")
        self.lblzitvl.setGeometry(QtCore.QRect(370, 170, 10, 30))
        self.cbbzitvl = QtWidgets.QComboBox(CropSurvey)
        self.cbbzitvl.setObjectName("cbbzitvl")
        self.cbbzitvl.setGeometry(QtCore.QRect(385, 170, 40, 30))
        self.btnapply = QtWidgets.QPushButton(CropSurvey)
        self.btnapply.setObjectName("btnapply")
        self.btnapply.setGeometry(QtCore.QRect(190, 230, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/ok.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnapply.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(CropSurvey)
        self.msgbox.setObjectName("msgbox")
        _center_x = CropSurvey.geometry().center().x()
        _center_y = CropSurvey.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(CropSurvey)
        QtCore.QMetaObject.connectSlotsByName(CropSurvey)


    def retranslateGUI(self, CropSurvey):
        self.dialog = CropSurvey
        #
        _translate = QtCore.QCoreApplication.translate
        CropSurvey.setWindowTitle(_translate("CropSurvey", "Crop Survey"))
        self.lblsrvinfo.setText(_translate("CropSurvey", "Survey information:"))
        self.lblstart.setText(_translate("CropSurvey", "Start"))
        self.lblstart.setAlignment(QtCore.Qt.AlignCenter)
        self.lblend.setText(_translate("CropSurvey", "End"))
        self.lblend.setAlignment(QtCore.Qt.AlignCenter)
        self.lblstep.setText(_translate("CropSurvey", "Step"))
        self.lblstep.setAlignment(QtCore.Qt.AlignCenter)
        self.lblinl.setText(_translate("CropSurvey", "Inline:"))
        self.lblinl.setAlignment(QtCore.Qt.AlignRight)
        self.lblxl.setText(_translate("CropSurvey", "Crossline:"))
        self.lblxl.setAlignment(QtCore.Qt.AlignRight)
        self.lblz.setText(_translate("CropSurvey", "Time/depth:"))
        self.lblz.setAlignment(QtCore.Qt.AlignRight)
        self.ldtinlstart.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtinlend.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtinlstep.setEnabled(False)
        self.ldtinlstep.setAlignment(QtCore.Qt.AlignCenter)
        self.lblinlitvl.setText(_translate("CropSurvey", "X"))
        self.cbbinlitvl.addItems([str(i+1) for i in range(100)])
        self.ldtxlstart.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtxlend.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtxlstep.setEnabled(False)
        self.ldtxlstep.setAlignment(QtCore.Qt.AlignCenter)
        self.lblxlitvl.setText(_translate("CropSurvey", "X"))
        self.cbbxlitvl.addItems([str(i + 1) for i in range(100)])
        self.ldtzstart.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtzend.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtzstep.setEnabled(False)
        self.ldtzstep.setAlignment(QtCore.Qt.AlignCenter)
        self.lblzitvl.setText(_translate("CropSurvey", "X"))
        self.cbbzitvl.addItems([str(i + 1) for i in range(100)])
        #
        if self.checkSurvInfo() is True:
            _survinfo = self.survinfo
            self.ldtinlstart.setText(_translate("CropSurvey", str(_survinfo['ILStart'])))
            self.ldtinlend.setText(_translate("CropSurvey", str(_survinfo['ILEnd'])))
            self.ldtinlstep.setText(_translate("CropSurvey", str(_survinfo['ILStep'])))
            self.ldtxlstart.setText(_translate("CropSurvey", str(_survinfo['XLStart'])))
            self.ldtxlend.setText(_translate("CropSurvey", str(_survinfo['XLEnd'])))
            self.ldtxlstep.setText(_translate("CropSurvey", str(_survinfo['XLStep'])))
            self.ldtzstart.setText(_translate("CropSurvey", str(_survinfo['ZStart'])))
            self.ldtzend.setText(_translate("CropSurvey", str(_survinfo['ZEnd'])))
            self.ldtzstep.setText(_translate("CropSurvey", str(_survinfo['ZStep'])))
        #
        self.btnapply.setText(_translate("CropSurvey", "Apply"))
        self.btnapply.clicked.connect(self.clickBtnApply)


    def clickBtnApply(self):
        self.refreshMsgBox()
        #
        if self.checkSurvInfo() is False:
            vis_msg.print("ERROR in CropSurvey: No seismic survey found", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Crop Survey',
                                           'No seismic survey found')
            return
        #
        _inlstart = basic_data.str2int(self.ldtinlstart.text())
        _xlstart = basic_data.str2int(self.ldtxlstart.text())
        _zstart = basic_data.str2int(self.ldtzstart.text())
        _inlend = basic_data.str2int(self.ldtinlend.text())
        _xlend = basic_data.str2int(self.ldtxlend.text())
        _zend = basic_data.str2int(self.ldtzend.text())
        if _inlstart is False or _xlstart is False or _zstart is False \
                or _inlend is False or _xlend is False or _zend is False:
            vis_msg.print("ERROR in CropSurvey: Non-integer survey selection", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Crop Survey',
                                           'Non-integer survey selection')
            return
        #
        _survinfo = self.survinfo
        #
        _inlstart_idx = _inlstart - _survinfo['ILStart']
        _inlstart_idx = int(_inlstart_idx / _survinfo['ILStep'])
        _xlstart_idx = _xlstart - _survinfo['XLStart']
        _xlstart_idx = int(_xlstart_idx / _survinfo['XLStep'])
        _zstart_idx = _zstart - _survinfo['ZStart']
        _zstart_idx = int(_zstart_idx / _survinfo['ZStep'])
        if _inlstart_idx < 0:
            _inlstart_idx = 0
        if _xlstart_idx < 0:
            _xlstart_idx = 0
        if _zstart_idx < 0:
            _zstart_idx = 0
        if _inlstart_idx >= _survinfo['ILNum']:
            _inlstart_idx = _survinfo['ILNum'] - 1
        if _xlstart_idx >= _survinfo['XLNum']:
            _xlstart_idx = _survinfo['XLNum'] - 1
        if _zstart_idx >= _survinfo['ZNum']:
            _zstart_idx = _survinfo['ZNum'] - 1
        _inlend_idx = _inlend - _survinfo['ILStart']
        _inlend_idx = int(_inlend_idx / _survinfo['ILStep'])
        _xlend_idx = _xlend - _survinfo['XLStart']
        _xlend_idx = int(_xlend_idx / _survinfo['XLStep'])
        _zend_idx = _zend - _survinfo['ZStart']
        _zend_idx = int(_zend_idx / _survinfo['ZStep'])
        if _inlend_idx >= _survinfo['ILNum']:
            _inlend_idx = _survinfo['ILNum'] - 1
        if _xlend_idx >= _survinfo['XLNum']:
            _xlend_idx = _survinfo['XLNum'] - 1
        if _zend_idx >= _survinfo['ZNum']:
            _zend_idx = _survinfo['ZNum'] - 1
        if _inlend_idx < _inlstart_idx:
            _inlend_idx = _inlstart_idx
        if _xlend_idx < _xlstart_idx:
            _xlend_idx = _xlstart_idx
        if _zend_idx < _zstart_idx:
            _zend_idx = _zstart_idx
        #
        _inlidx = np.arange(_inlstart_idx, _inlend_idx+1,
                            self.cbbinlitvl.currentIndex()+1, dtype=int)
        _xlidx = np.arange(_xlstart_idx, _xlend_idx+1,
                           self.cbbxlitvl.currentIndex() + 1, dtype=int)
        _zidx = np.arange(_zstart_idx, _zend_idx+1,
                          self.cbbzitvl.currentIndex() + 1, dtype=int)
        _idx = np.zeros([len(_inlidx), len(_xlidx), len(_zidx)])
        # survinfo
        _survinfo = seis_ays.createSeisInfoFrom3DMat(np.transpose(_idx, [2, 1, 0]),
                                                     inlstart=_inlstart_idx * self.survinfo['ILStep'] +
                                                              self.survinfo['ILStart'],
                                                     inlstep=(self.cbbinlitvl.currentIndex() + 1) *
                                                             self.survinfo['ILStep'],
                                                     xlstart=_xlstart_idx * self.survinfo['XLStep'] +
                                                             self.survinfo['XLStart'],
                                                     xlstep=(self.cbbxlitvl.currentIndex() + 1) *
                                                            self.survinfo['XLStep'],
                                                     zstart=_zstart_idx * self.survinfo['ZStep'] +
                                                            self.survinfo['ZStart'],
                                                     zstep=(self.cbbzitvl.currentIndex() + 1) *
                                                           self.survinfo['ZStep']
                                                     )
        # seisdata
        for i in range(len(_inlidx)):
            for j in range(len(_xlidx)):
                _idx[i, j, :] = _zidx + _xlidx[j] * self.survinfo['ZNum'] + \
                                _inlidx[i] * self.survinfo['XLNum'] * self.survinfo['ZNum']
        _idx = np.reshape(_idx, [len(_zidx) * len(_xlidx) * len(_inlidx)])
        for key in self.seisdata.keys():
            if self.checkSeisData(key):
                _dict = {}
                _dict[key] = np.reshape(np.transpose(self.seisdata[key], [2, 1, 0]), [-1, 1])
                self.seisdata[key] = np.transpose(np.reshape(basic_mdt.retrieveDictByIndex(_dict, _idx)[key],
                                                             [_survinfo['ILNum'],
                                                              _survinfo['XLNum'],
                                                              _survinfo['ZNum']]),
                                                  [2, 1, 0])
        #
        self.survinfo = _survinfo
        #
        QtWidgets.QMessageBox.information(self.msgbox,
                                          "Crop Survey",
                                          "Survey cropped successfully")
        #
        self.dialog.close()


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))

    def checkSurvInfo(self):
        self.refreshMsgBox()
        #
        if seis_ays.checkSeisInfo(self.survinfo) is False:
            # print("CropSurvey: Survey not found")
            # QtWidgets.QMessageBox.critical(self.msgbox,
            #                                'Crop Survey',
            #                                'Survey not found')
            return False
        return True

    def checkSeisData(self, f):
        self.refreshMsgBox()
        #
        return seis_ays.isSeis3DMatConsistentWithSeisInfo(self.seisdata[f], self.survinfo)


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    CropSurvey = QtWidgets.QWidget()
    gui = cropsurvey()
    gui.setupGUI(CropSurvey)
    CropSurvey.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# Create a window for managing seismic properties


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.gui.editseispointset import editseispointset as gui_editseispointset
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class manageseis(object):

    survinfo = {}
    seisdata = {}
    rootpath = ''
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, ManageSeis):
        ManageSeis.setObjectName("ManageSeis")
        ManageSeis.setFixedSize(320, 420)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/seismic.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ManageSeis.setWindowIcon(icon)
        #
        self.lblseis = QtWidgets.QLabel(ManageSeis)
        self.lblseis.setObjectName("lblattrib")
        self.lblseis.setGeometry(QtCore.QRect(10, 10, 150, 30))
        self.twgseis = QtWidgets.QTableWidget(ManageSeis)
        self.twgseis.setObjectName("twgseis")
        self.twgseis.setGeometry(QtCore.QRect(10, 50, 300, 300))
        self.twgseis.setColumnCount(5)
        self.twgseis.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.twgseis.verticalHeader().hide()
        self.btnedit = QtWidgets.QPushButton(ManageSeis)
        self.btnedit.setObjectName("btnedit")
        self.btnedit.setGeometry(QtCore.QRect(210, 360, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/pen.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnedit.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(ManageSeis)
        self.msgbox.setObjectName("msgbox")
        _center_x = ManageSeis.geometry().center().x()
        _center_y = ManageSeis.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ManageSeis)
        QtCore.QMetaObject.connectSlotsByName(ManageSeis)


    def retranslateGUI(self, ManageSeis):
        self.dialog = ManageSeis
        #
        _translate = QtCore.QCoreApplication.translate
        ManageSeis.setWindowTitle(_translate("ManageSeis", "Manage Seismic"))
        self.lblseis.setText(_translate("ManageSeis", "Seismic properties:"))
        self.refreshTwgSeis()
        self.btnedit.setText(_translate("ManageSeis", "Edit"))
        self.btnedit.setToolTip("Edit seismic properties")
        self.btnedit.clicked.connect(self.clickBtnEdit)


    def clickBtnEdit(self):
        _editseis = QtWidgets.QDialog()
        _gui = gui_editseispointset()
        _gui.seispointdata = {}
        # add info
        if self.checkSurvInfo():
            _survinfo = seis_ays.convertSeisInfoTo2DMat(self.survinfo)
            _gui.seispointdata['Inline'] = _survinfo[:, 0:1]
            _gui.seispointdata['Crossline'] = _survinfo[:, 1:2]
            _gui.seispointdata['Z'] = _survinfo[:, 2:3]
        # reshape data
        for key in self.seisdata.keys():
            if self.checkSeisData(key):
                _gui.seispointdata[key] = np.reshape(np.transpose(self.seisdata[key], [2, 1, 0]), [-1, 1])
        _gui.rootpath = self.rootpath
        _gui.setupGUI(_editseis)
        _editseis.exec()
        #
        self.seisdata = {}
        # reshape data
        for key in _gui.seispointdata.keys():
            if key != 'Inline' and key != 'Crossline' and key != 'Z' \
                    and (np.shape(_gui.seispointdata[key])[0] == self.survinfo['ILNum'] * self.survinfo['XLNum'] * self.survinfo['ZNum']):
                self.seisdata[key] = np.transpose(np.reshape(_gui.seispointdata[key], [self.survinfo['ILNum'],
                                                                                       self.survinfo['XLNum'],
                                                                                       self.survinfo['ZNum']]),
                                                  [2, 1, 0])
        _editseis.show()
        #
        self.refreshTwgSeis()


    def refreshTwgSeis(self):
        self.twgseis.clear()
        self.twgseis.setHorizontalHeaderLabels(["Property", "Maximum", "Minimum", "Mean", "Std"])
        if self.checkSurvInfo() is True:
            _idx = 0
            self.twgseis.setRowCount(len(self.seisdata.keys()))
            for i in sorted(self.seisdata.keys()):
                if self.checkSeisData(i):
                    self.twgseis.setRowCount(_idx+1)
                    #
                    item = QtWidgets.QTableWidgetItem()
                    item.setText(i)
                    item.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                    self.twgseis.setItem(_idx, 0, item)
                    item = QtWidgets.QTableWidgetItem()
                    item.setText(str(np.max(self.seisdata[i])))
                    item.setTextAlignment(QtCore.Qt.AlignCenter)
                    item.setFlags(QtCore.Qt.ItemIsEditable)
                    self.twgseis.setItem(_idx, 1, item)
                    item = QtWidgets.QTableWidgetItem()
                    item.setText(str(np.min(self.seisdata[i])))
                    item.setTextAlignment(QtCore.Qt.AlignCenter)
                    item.setFlags(QtCore.Qt.ItemIsEditable)
                    self.twgseis.setItem(_idx, 2, item)
                    item = QtWidgets.QTableWidgetItem()
                    item.setText(str(np.mean(self.seisdata[i])))
                    item.setTextAlignment(QtCore.Qt.AlignCenter)
                    item.setFlags(QtCore.Qt.ItemIsEditable)
                    self.twgseis.setItem(_idx, 3, item)
                    item = QtWidgets.QTableWidgetItem()
                    item.setText(str(np.std(self.seisdata[i])))
                    item.setTextAlignment(QtCore.Qt.AlignCenter)
                    item.setFlags(QtCore.Qt.ItemIsEditable)
                    self.twgseis.setItem(_idx, 4, item)
                    _idx = _idx + 1


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkSurvInfo(self):
        self.refreshMsgBox()
        #
        if seis_ays.checkSeisInfo(self.survinfo) is False:
            # print("ManageSeis: Survey not found")
            # QtWidgets.QMessageBox.critical(self.msgbox,
            #                                'Manage Seismic',
            #                                'Survey not found')
            return False
        return True


    def checkSeisData(self, f):
        self.refreshMsgBox()
        #
        return seis_ays.isSeis3DMatConsistentWithSeisInfo(self.seisdata[f], self.survinfo)



if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ManageSeis = QtWidgets.QWidget()
    gui = manageseis()
    gui.setupGUI(ManageSeis)
    ManageSeis.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# basic functions for processing video

import sys, os
import numpy as np
from scipy import interpolate
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.vis.messager import messager as vis_msg


__all__ = ['video']


def changeVideoSize(video, video_height, video_width, video_depth,
                    video_height_new, video_width_new, video_depth_new, method='linear'):
    """
    Change video size through 3D interpolation (RegularGridInterpolator from scipy)

    Args:
        video:              2D matrix of videos [pixel 1, pixel 2, ..., pixel N]
                            Each row for all pixels in an video
        video_height:       Original height of video
        video_width:        Original width of video
        video_depth:        Original depth of video
        video_height_new:   New height of video
        video_width_new:    New width of video
        video_depth_new:    New depth of video
        method:             Interpolation kind, 'linear' or 'nearest'. Default is 'linear'

    Returns:
        2D matrix of videos after interpolation, with each row representing one video
    """

    if np.ndim(video) != 2:
        vis_msg.print('ERROR in changeVideoSize: 2D video matrix expected', type='error')
        sys.exit()
    if video_height <= 1 or video_width <= 1 or video_depth <= 1:
        vis_msg.print('ERROR in changeVideoSize: Original video height/width/depth be > 1', type='error')
        sys.exit()
    if video_height_new <= 0 or video_width_new <= 0 or video_depth_new <= 0:
        vis_msg.print('ERROR in changeVideoSize: New video height/width/depth be >= 1', type='error')
        sys.exit()

    nvideo, npixel = np.shape(video)

    if npixel != video_height * video_width * video_depth:
        vis_msg.print('ERROR in changeVideoSize: Original video height/width/depth not match', type='error')
        sys.exit()

    height = np.linspace(0.0, 1.0, video_height)
    width = np.linspace(0.0, 1.0, video_width)
    depth = np.linspace(0.0, 1.0, video_depth)
    height_new = np.linspace(0.0, 1.0, video_height_new)
    width_new = np.linspace(0.0, 1.0, video_width_new)
    depth_new = np.linspace(0.0, 1.0, video_depth_new)
    npixel_new = video_height_new * video_width_new * video_depth_new
    height_new, width_new, depth_new = np.meshgrid(height_new, width_new, depth_new, indexing='ij')
    height_new = np.reshape(height_new, [npixel_new, 1])
    width_new = np.reshape(width_new, [npixel_new, 1])
    depth_new = np.reshape(depth_new, [npixel_new, 1])

    video_new = np.zeros([nvideo, npixel_new])
    for i in range(nvideo):
        video_i = video[i, :]
        video_i = np.reshape(video_i, [video_height, video_width, video_depth])

        f = interpolate.RegularGridInterpolator((height, width, depth), video_i, method=method)
        video_i_new = f(np.concatenate((height_new, width_new, depth_new), axis=1))
        video_i_new = np.reshape(video_i_new, [video_height_new, video_width_new, video_depth_new])
        video_new[i, :] = np.reshape(video_i_new, [1, npixel_new])

    return video_new


class video:
    # Pack all functions as a class
    #
    changeVideoSize = changeVideoSize


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# Create a window for managing seismic survey


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.gui.editsurvey import editsurvey as gui_editsurvey
from cognitivegeo.src.gui.cropsurvey import cropsurvey as gui_cropsurvey
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class managesurvey(object):

    survinfo = {}
    seisdata = {}
    #
    iconpath = os.path.dirname(__file__)
    dialog = None

    def setupGUI(self, ManageSurvey):
        ManageSurvey.setObjectName("ManageSurvey")
        ManageSurvey.setFixedSize(500, 330)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/survey.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ManageSurvey.setWindowIcon(icon)
        self.lblsrvinfo = QtWidgets.QLabel(ManageSurvey)
        self.lblsrvinfo.setObjectName("lblsrvinfo")
        self.lblsrvinfo.setGeometry(QtCore.QRect(10, 10, 150, 30))
        self.lblstart = QtWidgets.QLabel(ManageSurvey)
        self.lblstart.setObjectName("lblstart")
        self.lblstart.setGeometry(QtCore.QRect(120, 50, 80, 30))
        self.lblend = QtWidgets.QLabel(ManageSurvey)
        self.lblend.setObjectName("lblend")
        self.lblend.setGeometry(QtCore.QRect(220, 50, 80, 30))
        self.lblstep = QtWidgets.QLabel(ManageSurvey)
        self.lblstep.setObjectName("lblstep")
        self.lblstep.setGeometry(QtCore.QRect(320, 50, 40, 30))
        self.lblnum = QtWidgets.QLabel(ManageSurvey)
        self.lblnum.setObjectName("lblnum")
        self.lblnum.setGeometry(QtCore.QRect(380, 50, 80, 30))
        self.lblinl = QtWidgets.QLabel(ManageSurvey)
        self.lblinl.setObjectName("lblinl")
        self.lblinl.setGeometry(QtCore.QRect(10, 90, 100, 30))
        self.lblxl = QtWidgets.QLabel(ManageSurvey)
        self.lblxl.setObjectName("lblxl")
        self.lblxl.setGeometry(QtCore.QRect(10, 130, 100, 30))
        self.lblz = QtWidgets.QLabel(ManageSurvey)
        self.lblz.setObjectName("lblz")
        self.lblz.setGeometry(QtCore.QRect(10, 170, 100, 30))
        self.ldtinlstart = QtWidgets.QLineEdit(ManageSurvey)
        self.ldtinlstart.setObjectName("ldtinlstart")
        self.ldtinlstart.setGeometry(QtCore.QRect(120, 90, 80, 30))
        self.ldtinlend = QtWidgets.QLineEdit(ManageSurvey)
        self.ldtinlend.setObjectName("ldtinlend")
        self.ldtinlend.setGeometry(QtCore.QRect(220, 90, 80, 30))
        self.ldtinlstep = QtWidgets.QLineEdit(ManageSurvey)
        self.ldtinlstep.setObjectName("ldtinlstep")
        self.ldtinlstep.setGeometry(QtCore.QRect(320, 90, 40, 30))
        self.ldtinlnum = QtWidgets.QLineEdit(ManageSurvey)
        self.ldtinlnum.setObjectName("ldtinlnum")
        self.ldtinlnum.setGeometry(QtCore.QRect(380, 90, 80, 30))
        self.ldtxlstart = QtWidgets.QLineEdit(ManageSurvey)
        self.ldtxlstart.setObjectName("ldtxlstart")
        self.ldtxlstart.setGeometry(QtCore.QRect(120, 130, 80, 30))
        self.ldtxlend = QtWidgets.QLineEdit(ManageSurvey)
        self.ldtxlend.setObjectName("ldtxlend")
        self.ldtxlend.setGeometry(QtCore.QRect(220, 130, 80, 30))
        self.ldtxlstep = QtWidgets.QLineEdit(ManageSurvey)
        self.ldtxlstep.setObjectName("ldtxlstep")
        self.ldtxlstep.setGeometry(QtCore.QRect(320, 130, 40, 30))
        self.ldtxlnum = QtWidgets.QLineEdit(ManageSurvey)
        self.ldtxlnum.setObjectName("ldtxlnum")
        self.ldtxlnum.setGeometry(QtCore.QRect(380, 130, 80, 30))
        self.ldtzstart = QtWidgets.QLineEdit(ManageSurvey)
        self.ldtzstart.setObjectName("ldtzstart")
        self.ldtzstart.setGeometry(QtCore.QRect(120, 170, 80, 30))
        self.ldtzend = QtWidgets.QLineEdit(ManageSurvey)
        self.ldtzend.setObjectName("ldtzend")
        self.ldtzend.setGeometry(QtCore.QRect(220, 170, 80, 30))
        self.ldtzstep = QtWidgets.QLineEdit(ManageSurvey)
        self.ldtzstep.setObjectName("ldtzlstep")
        self.ldtzstep.setGeometry(QtCore.QRect(320, 170, 40, 30))
        self.ldtznum = QtWidgets.QLineEdit(ManageSurvey)
        self.ldtznum.setObjectName("ldtznum")
        self.ldtznum.setGeometry(QtCore.QRect(380, 170, 80, 30))
        self.btnedit = QtWidgets.QPushButton(ManageSurvey)
        self.btnedit.setObjectName("btnedit")
        self.btnedit.setGeometry(QtCore.QRect(360, 220, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/pen.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnedit.setIcon(icon)
        self.btncrop = QtWidgets.QPushButton(ManageSurvey)
        self.btncrop.setObjectName("btncrop")
        self.btncrop.setGeometry(QtCore.QRect(360, 270, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/crop.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btncrop.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(ManageSurvey)
        self.msgbox.setObjectName("msgbox")
        _center_x = ManageSurvey.geometry().center().x()
        _center_y = ManageSurvey.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ManageSurvey)
        QtCore.QMetaObject.connectSlotsByName(ManageSurvey)


    def retranslateGUI(self, ManageSurvey):
        self.dialog = ManageSurvey
        #
        _translate = QtCore.QCoreApplication.translate
        ManageSurvey.setWindowTitle(_translate("ManageSurvey", "Manage Survey"))
        self.lblsrvinfo.setText(_translate("ManageSurvey", "Survey information:"))
        self.lblstart.setText(_translate("ManageSurvey", "Start"))
        self.lblstart.setAlignment(QtCore.Qt.AlignCenter)
        self.lblend.setText(_translate("ManageSurvey", "End"))
        self.lblend.setAlignment(QtCore.Qt.AlignCenter)
        self.lblstep.setText(_translate("ManageSurvey", "Step"))
        self.lblstep.setAlignment(QtCore.Qt.AlignCenter)
        self.lblnum.setText(_translate("ManageSurvey", "Number"))
        self.lblnum.setAlignment(QtCore.Qt.AlignCenter)
        self.lblinl.setText(_translate("ManageSurvey", "Inline:"))
        self.lblinl.setAlignment(QtCore.Qt.AlignRight)
        self.lblxl.setText(_translate("ManageSurvey", "Crossline:"))
        self.lblxl.setAlignment(QtCore.Qt.AlignRight)
        self.lblz.setText(_translate("ManageSurvey", "Time/depth:"))
        self.lblz.setAlignment(QtCore.Qt.AlignRight)
        self.ldtinlstart.setEnabled(False)
        self.ldtinlstart.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtinlend.setEnabled(False)
        self.ldtinlend.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtinlstep.setEnabled(False)
        self.ldtinlstep.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtinlnum.setEnabled(False)
        self.ldtinlnum.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtxlstart.setEnabled(False)
        self.ldtxlstart.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtxlend.setEnabled(False)
        self.ldtxlend.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtxlstep.setEnabled(False)
        self.ldtxlstep.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtxlnum.setEnabled(False)
        self.ldtxlnum.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtzstart.setEnabled(False)
        self.ldtzstart.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtzend.setEnabled(False)
        self.ldtzend.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtzstep.setEnabled(False)
        self.ldtzstep.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtznum.setEnabled(False)
        self.ldtznum.setAlignment(QtCore.Qt.AlignCenter)
        #
        self.refreshSeisInfo()
        #
        self.btnedit.setText(_translate("ManageSurvey", "Edit"))
        self.btnedit.setToolTip("Edit survey range")
        self.btnedit.clicked.connect(self.clickBtnEdit)
        self.btncrop.setText(_translate("ManageSurvey", "Crop"))
        self.btncrop.setToolTip("Crop survey size")
        self.btncrop.clicked.connect(self.clickBtnCrop)


    def clickBtnEdit(self):
        _editsurvey = QtWidgets.QDialog()
        _gui = gui_editsurvey()
        _gui.survinfo = self.survinfo
        _gui.setupGUI(_editsurvey)
        _editsurvey.exec()
        self.survinfo = _gui.survinfo
        _editsurvey.show()
        #
        self.refreshSeisInfo()


    def clickBtnCrop(self):
        _cropsurvey = QtWidgets.QDialog()
        _gui = gui_cropsurvey()
        _gui.survinfo = self.survinfo
        _gui.seisdata = self.seisdata
        _gui.setupGUI(_cropsurvey)
        _cropsurvey.exec()
        self.seisdata = _gui.seisdata
        self.survinfo = _gui.survinfo
        _cropsurvey.show()
        #
        self.refreshSeisInfo()


    def refreshSeisInfo(self):
        if (self.checkSurvInfo() is True):
            _seisinfo = self.survinfo
            self.ldtinlstart.setText(str(_seisinfo['ILStart']))
            self.ldtinlend.setText(str(_seisinfo['ILEnd']))
            self.ldtinlstep.setText(str(_seisinfo['ILStep']))
            self.ldtinlnum.setText(str(_seisinfo['ILNum']))
            self.ldtxlstart.setText(str(_seisinfo['XLStart']))
            self.ldtxlend.setText(str(_seisinfo['XLEnd']))
            self.ldtxlstep.setText(str(_seisinfo['XLStep']))
            self.ldtxlnum.setText(str(_seisinfo['XLNum']))
            self.ldtzstart.setText(str(_seisinfo['ZStart']))
            self.ldtzend.setText(str(_seisinfo['ZEnd']))
            self.ldtzstep.setText(str(_seisinfo['ZStep']))
            self.ldtznum.setText(str(_seisinfo['ZNum']))


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkSurvInfo(self):
        self.refreshMsgBox()
        #
        if seis_ays.checkSeisInfo(self.survinfo) is False:
            # print("ManageSurvey: Survey not found")
            # QtWidgets.QMessageBox.critical(self.msgbox,
            #                                'Manage Survey',
            #                                'Survey not found')
            return False
        return True


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    ManageSurvey = QtWidgets.QWidget()
    gui = managesurvey()
    gui.setupGUI(ManageSurvey)
    ManageSurvey.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     February 2019                                                                   #
#                                                                                           #
#############################################################################################

# Create a window for cropping a survey into pre-stack seismic


from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.basic.data import data as basic_data
from cognitivegeo.src.seismic.analysis import analysis as seis_ays
from cognitivegeo.src.psseismic.analysis import analysis as psseis_ays
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class convertseis2psseis(object):

    survinfo = {}
    seisdata = {}
    psseisdata = {}
    #
    iconpath = os.path.dirname(__file__)
    dialog = None

    def setupGUI(self, ConvertSeis2PsSeis):
        ConvertSeis2PsSeis.setObjectName("ConvertSeis2PsSeis")
        ConvertSeis2PsSeis.setFixedSize(440, 450)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/seismic.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        ConvertSeis2PsSeis.setWindowIcon(icon)
        self.lblsrvinfo = QtWidgets.QLabel(ConvertSeis2PsSeis)
        self.lblsrvinfo.setObjectName("lblsrvinfo")
        self.lblsrvinfo.setGeometry(QtCore.QRect(10, 10, 150, 30))
        self.rdbsrvall = QtWidgets.QRadioButton(ConvertSeis2PsSeis)
        self.rdbsrvall.setObjectName("rdbsrvall")
        self.rdbsrvall.setGeometry(QtCore.QRect(130, 10, 150, 30))
        self.rdbsrvpart = QtWidgets.QRadioButton(ConvertSeis2PsSeis)
        self.rdbsrvpart.setObjectName("rdbsrvpart")
        self.rdbsrvpart.setGeometry(QtCore.QRect(280, 10, 150, 30))
        self.lblstart = QtWidgets.QLabel(ConvertSeis2PsSeis)
        self.lblstart.setObjectName("lblstart")
        self.lblstart.setGeometry(QtCore.QRect(120, 50, 80, 30))
        self.lblend = QtWidgets.QLabel(ConvertSeis2PsSeis)
        self.lblend.setObjectName("lblend")
        self.lblend.setGeometry(QtCore.QRect(220, 50, 80, 30))
        self.lblstep = QtWidgets.QLabel(ConvertSeis2PsSeis)
        self.lblstep.setObjectName("lblstep")
        self.lblstep.setGeometry(QtCore.QRect(320, 50, 40, 30))
        self.lblinl = QtWidgets.QLabel(ConvertSeis2PsSeis)
        self.lblinl.setObjectName("lblinl")
        self.lblinl.setGeometry(QtCore.QRect(10, 90, 100, 30))
        self.lblxl = QtWidgets.QLabel(ConvertSeis2PsSeis)
        self.lblxl.setObjectName("lblxl")
        self.lblxl.setGeometry(QtCore.QRect(10, 130, 100, 30))
        self.lblz = QtWidgets.QLabel(ConvertSeis2PsSeis)
        self.lblz.setObjectName("lblz")
        self.lblz.setGeometry(QtCore.QRect(10, 170, 100, 30))
        self.ldtinlstart = QtWidgets.QLineEdit(ConvertSeis2PsSeis)
        self.ldtinlstart.setObjectName("ldtinlstart")
        self.ldtinlstart.setGeometry(QtCore.QRect(120, 90, 80, 30))
        self.ldtinlend = QtWidgets.QLineEdit(ConvertSeis2PsSeis)
        self.ldtinlend.setObjectName("ldtinlend")
        self.ldtinlend.setGeometry(QtCore.QRect(220, 90, 80, 30))
        self.ldtinlstep = QtWidgets.QLineEdit(ConvertSeis2PsSeis)
        self.ldtinlstep.setObjectName("ldtinlstep")
        self.ldtinlstep.setGeometry(QtCore.QRect(320, 90, 40, 30))
        self.lblinlitvl = QtWidgets.QLabel(ConvertSeis2PsSeis)
        self.lblinlitvl.setObjectName("lblinlitvl")
        self.lblinlitvl.setGeometry(QtCore.QRect(370, 90, 10, 30))
        self.cbbinlitvl = QtWidgets.QComboBox(ConvertSeis2PsSeis)
        self.cbbinlitvl.setObjectName("cbbinlitvl")
        self.cbbinlitvl.setGeometry(QtCore.QRect(385, 90, 45, 30))
        self.ldtxlstart = QtWidgets.QLineEdit(ConvertSeis2PsSeis)
        self.ldtxlstart.setObjectName("ldtxlstart")
        self.ldtxlstart.setGeometry(QtCore.QRect(120, 130, 80, 30))
        self.ldtxlend = QtWidgets.QLineEdit(ConvertSeis2PsSeis)
        self.ldtxlend.setObjectName("ldtxlend")
        self.ldtxlend.setGeometry(QtCore.QRect(220, 130, 80, 30))
        self.ldtxlstep = QtWidgets.QLineEdit(ConvertSeis2PsSeis)
        self.ldtxlstep.setObjectName("ldtxlstep")
        self.ldtxlstep.setGeometry(QtCore.QRect(320, 130, 40, 30))
        self.lblxlitvl = QtWidgets.QLabel(ConvertSeis2PsSeis)
        self.lblxlitvl.setObjectName("lblxlitvl")
        self.lblxlitvl.setGeometry(QtCore.QRect(370, 130, 10, 30))
        self.cbbxlitvl = QtWidgets.QComboBox(ConvertSeis2PsSeis)
        self.cbbxlitvl.setObjectName("cbbxlitvl")
        self.cbbxlitvl.setGeometry(QtCore.QRect(385, 130, 45, 30))
        self.ldtzstart = QtWidgets.QLineEdit(ConvertSeis2PsSeis)
        self.ldtzstart.setObjectName("ldtzstart")
        self.ldtzstart.setGeometry(QtCore.QRect(120, 170, 80, 30))
        self.ldtzend = QtWidgets.QLineEdit(ConvertSeis2PsSeis)
        self.ldtzend.setObjectName("ldtzend")
        self.ldtzend.setGeometry(QtCore.QRect(220, 170, 80, 30))
        self.ldtzstep = QtWidgets.QLineEdit(ConvertSeis2PsSeis)
        self.ldtzstep.setObjectName("ldtzlstep")
        self.ldtzstep.setGeometry(QtCore.QRect(320, 170, 40, 30))
        self.lblzitvl = QtWidgets.QLabel(ConvertSeis2PsSeis)
        self.lblzitvl.setObjectName("lblzitvl")
        self.lblzitvl.setGeometry(QtCore.QRect(370, 170, 10, 30))
        self.cbbzitvl = QtWidgets.QComboBox(ConvertSeis2PsSeis)
        self.cbbzitvl.setObjectName("cbbzitvl")
        self.cbbzitvl.setGeometry(QtCore.QRect(385, 170, 45, 30))
        #
        self.lblattrib = QtWidgets.QLabel(ConvertSeis2PsSeis)
        self.lblattrib.setObjectName("lblattrib")
        self.lblattrib.setGeometry(QtCore.QRect(10, 210, 150, 30))
        self.lwgattrib = QtWidgets.QListWidget(ConvertSeis2PsSeis)
        self.lwgattrib.setObjectName("lwgattrib")
        self.lwgattrib.setGeometry(QtCore.QRect(10, 250, 220, 180))
        self.lwgattrib.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        ##
        self.lblsave = QtWidgets.QLabel(ConvertSeis2PsSeis)
        self.lblsave.setObjectName("lblsave")
        self.lblsave.setGeometry(QtCore.QRect(250, 350, 70, 30))
        self.ldtsave = QtWidgets.QLineEdit(ConvertSeis2PsSeis)
        self.ldtsave.setObjectName("ldtsave")
        self.ldtsave.setGeometry(QtCore.QRect(330, 350, 100, 30))
        #
        self.btnapply = QtWidgets.QPushButton(ConvertSeis2PsSeis)
        self.btnapply.setObjectName("btnapply")
        self.btnapply.setGeometry(QtCore.QRect(330, 400, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/ok.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnapply.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(ConvertSeis2PsSeis)
        self.msgbox.setObjectName("msgbox")
        _center_x = ConvertSeis2PsSeis.geometry().center().x()
        _center_y = ConvertSeis2PsSeis.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(ConvertSeis2PsSeis)
        QtCore.QMetaObject.connectSlotsByName(ConvertSeis2PsSeis)


    def retranslateGUI(self, ConvertSeis2PsSeis):
        self.dialog = ConvertSeis2PsSeis
        #
        _translate = QtCore.QCoreApplication.translate
        ConvertSeis2PsSeis.setWindowTitle(_translate("ConvertSeis2PsSeis", "Convert Seismic to Pre-stack"))
        self.lblsrvinfo.setText(_translate("ConvertSeis2PsSeis", "Select survey:"))
        self.rdbsrvall.setText(_translate("ExportSeisSegy", "All"))
        self.rdbsrvall.setChecked(False)
        self.rdbsrvall.clicked.connect(self.clickRdbSrvAll)
        self.rdbsrvpart.setText(_translate("ExportSeisSegy", "Customize"))
        self.rdbsrvpart.setChecked(True)
        self.rdbsrvpart.clicked.connect(self.clickRdbSrvPart)
        self.lblstart.setText(_translate("ConvertSeis2PsSeis", "Start"))
        self.lblstart.setAlignment(QtCore.Qt.AlignCenter)
        self.lblend.setText(_translate("ConvertSeis2PsSeis", "End"))
        self.lblend.setAlignment(QtCore.Qt.AlignCenter)
        self.lblstep.setText(_translate("ConvertSeis2PsSeis", "Step"))
        self.lblstep.setAlignment(QtCore.Qt.AlignCenter)
        self.lblinl.setText(_translate("ConvertSeis2PsSeis", "Inline:"))
        self.lblinl.setAlignment(QtCore.Qt.AlignRight)
        self.lblxl.setText(_translate("ConvertSeis2PsSeis", "Crossline:"))
        self.lblxl.setAlignment(QtCore.Qt.AlignRight)
        self.lblz.setText(_translate("ConvertSeis2PsSeis", "Time/depth:"))
        self.lblz.setAlignment(QtCore.Qt.AlignRight)
        self.ldtinlstart.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtinlend.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtinlstep.setEnabled(False)
        self.ldtinlstep.setAlignment(QtCore.Qt.AlignCenter)
        self.lblinlitvl.setText(_translate("ConvertSeis2PsSeis", "X"))
        self.cbbinlitvl.addItems([str(i+1) for i in range(100)])
        self.ldtxlstart.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtxlend.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtxlstep.setEnabled(False)
        self.ldtxlstep.setAlignment(QtCore.Qt.AlignCenter)
        self.lblxlitvl.setText(_translate("ConvertSeis2PsSeis", "X"))
        self.cbbxlitvl.addItems([str(i + 1) for i in range(100)])
        self.ldtzstart.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtzend.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtzstep.setEnabled(False)
        self.ldtzstep.setAlignment(QtCore.Qt.AlignCenter)
        self.lblzitvl.setText(_translate("ConvertSeis2PsSeis", "X"))
        self.cbbzitvl.addItems([str(i + 1) for i in range(100)])
        #
        self.lblattrib.setText(_translate("ConvertSeis2PsSeis", "Select properties:"))
        #
        if self.checkSurvInfo() is True:
            _survinfo = self.survinfo
            self.ldtinlstart.setText(_translate("ConvertSeis2PsSeis", str(_survinfo['ILStart'])))
            self.ldtinlend.setText(_translate("ConvertSeis2PsSeis", str(_survinfo['ILEnd'])))
            self.ldtinlstep.setText(_translate("ConvertSeis2PsSeis", str(_survinfo['ILStep'])))
            self.ldtxlstart.setText(_translate("ConvertSeis2PsSeis", str(_survinfo['XLStart'])))
            self.ldtxlend.setText(_translate("ConvertSeis2PsSeis", str(_survinfo['XLEnd'])))
            self.ldtxlstep.setText(_translate("ConvertSeis2PsSeis", str(_survinfo['XLStep'])))
            self.ldtzstart.setText(_translate("ConvertSeis2PsSeis", str(_survinfo['ZStart'])))
            self.ldtzend.setText(_translate("ConvertSeis2PsSeis", str(_survinfo['ZEnd'])))
            self.ldtzstep.setText(_translate("ConvertSeis2PsSeis", str(_survinfo['ZStep'])))
            #
            for i in sorted(self.seisdata.keys()):
                if self.checkSeisData(i):
                    item = QtWidgets.QListWidgetItem(self.lwgattrib)
                    item.setText(i)
                    self.lwgattrib.addItem(item)
            # self.lwgattrib.selectAll()
        #
        self.lblsave.setText(_translate("ConvertSeis2PsSeis", "Save as"))
        self.lblsave.setAlignment(QtCore.Qt.AlignRight)
        self.ldtsave.setText(_translate("ConvertSeis2PsSeis", 'prestack'))
        self.ldtsave.setAlignment(QtCore.Qt.AlignCenter)
        #
        self.btnapply.setText(_translate("ConvertSeis2PsSeis", "Apply"))
        self.btnapply.clicked.connect(self.clickBtnApply)


    def clickRdbSrvAll(self):
        if self.rdbsrvall.isChecked():
            self.rdbsrvall.setChecked(True)
            self.rdbsrvpart.setChecked(False)
            #
            self.cbbinlitvl.setCurrentIndex(0)
            self.cbbxlitvl.setCurrentIndex(0)
            self.cbbzitvl.setCurrentIndex(0)
            #
            self.ldtinlstart.setEnabled(False)
            self.ldtinlend.setEnabled(False)
            self.ldtxlstart.setEnabled(False)
            self.ldtxlend.setEnabled(False)
            self.ldtzstart.setEnabled(False)
            self.ldtzend.setEnabled(False)
            self.cbbinlitvl.setEnabled(False)
            self.cbbxlitvl.setEnabled(False)
            self.cbbzitvl.setEnabled(False)
            #
            if self.checkSurvInfo() is True:
                _survinfo = self.survinfo
                self.ldtinlstart.setText(str(_survinfo['ILStart']))
                self.ldtinlend.setText(str(_survinfo['ILEnd']))
                self.ldtinlstep.setText(str(_survinfo['ILStep']))
                self.ldtxlstart.setText(str(_survinfo['XLStart']))
                self.ldtxlend.setText(str(_survinfo['XLEnd']))
                self.ldtxlstep.setText(str(_survinfo['XLStep']))
                self.ldtzstart.setText(str(_survinfo['ZStart']))
                self.ldtzend.setText(str(_survinfo['ZEnd']))
                self.ldtzstep.setText(str(_survinfo['ZStep']))


    def clickRdbSrvPart(self):
        if self.rdbsrvpart.isChecked():
            self.rdbsrvpart.setChecked(True)
            self.rdbsrvall.setChecked(False)
            #
            self.ldtinlstart.setEnabled(True)
            self.ldtinlend.setEnabled(True)
            self.ldtxlstart.setEnabled(True)
            self.ldtxlend.setEnabled(True)
            self.ldtzstart.setEnabled(True)
            self.ldtzend.setEnabled(True)
            self.cbbinlitvl.setEnabled(True)
            self.cbbxlitvl.setEnabled(True)
            self.cbbzitvl.setEnabled(True)


    def clickBtnApply(self):
        self.refreshMsgBox()
        #
        if self.ldtsave.text() in self.psseisdata.keys():
            reply = QtWidgets.QMessageBox.question(self.msgbox, 'Convert Seismic to Pre-stack',
                                                   self.ldtsave.text() + ' already exists. Overwrite?',
                                                   QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No,
                                                   QtWidgets.QMessageBox.No)

            if reply == QtWidgets.QMessageBox.No:
                return
        #
        if self.checkSurvInfo() is False:
            vis_msg.print("ERROR in ConvertSeis2PsSeis: No seismic survey found", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Convert Seismic to Pre-stack',
                                           'No seismic survey found')
            return
        #
        if len(self.lwgattrib.selectedItems()) < 1:
            vis_msg.print("ERROR in ConvertSeis2PsSeis: No seismic property selected", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'Convert Seismic to Pre-stack',
                                           'No seismic property selected')
            return
        #
        if self.rdbsrvpart.isChecked():
            _inlstart = basic_data.str2int(self.ldtinlstart.text())
            _xlstart = basic_data.str2int(self.ldtxlstart.text())
            _zstart = basic_data.str2int(self.ldtzstart.text())
            _inlend = basic_data.str2int(self.ldtinlend.text())
            _xlend = basic_data.str2int(self.ldtxlend.text())
            _zend = basic_data.str2int(self.ldtzend.text())
            if _inlstart is False or _xlstart is False or _zstart is False \
                    or _inlend is False or _xlend is False or _zend is False:
                vis_msg.print("ERROR in ConvertSeis2PsSeis: Non-integer survey selection", type='error')
                QtWidgets.QMessageBox.critical(self.msgbox,
                                               'Convert Seismic to Pre-stack',
                                               'Non-integer survey selection')
                return
        #
        _survinfo = self.survinfo
        #
        if self.rdbsrvall.isChecked():
            _pts = seis_ays.convertSeisInfoTo2DMat(self.survinfo)
        if self.rdbsrvpart.isChecked():
            _inlstart_idx = _inlstart - _survinfo['ILStart']
            _inlstart_idx = int(_inlstart_idx / _survinfo['ILStep'])
            _xlstart_idx = _xlstart - _survinfo['XLStart']
            _xlstart_idx = int(_xlstart_idx / _survinfo['XLStep'])
            _zstart_idx = _zstart - _survinfo['ZStart']
            _zstart_idx = int(_zstart_idx / _survinfo['ZStep'])
            if _inlstart_idx < 0:
                _inlstart_idx = 0
            if _xlstart_idx < 0:
                _xlstart_idx = 0
            if _zstart_idx < 0:
                _zstart_idx = 0
            if _inlstart_idx >= _survinfo['ILNum']:
                _inlstart_idx = _survinfo['ILNum'] - 1
            if _xlstart_idx >= _survinfo['XLNum']:
                _xlstart_idx = _survinfo['XLNum'] - 1
            if _zstart_idx >= _survinfo['ZNum']:
                _zstart_idx = _survinfo['ZNum'] - 1
            _inlend_idx = _inlend - _survinfo['ILStart']
            _inlend_idx = int(_inlend_idx / _survinfo['ILStep'])
            _xlend_idx = _xlend - _survinfo['XLStart']
            _xlend_idx = int(_xlend_idx / _survinfo['XLStep'])
            _zend_idx = _zend - _survinfo['ZStart']
            _zend_idx = int(_zend_idx / _survinfo['ZStep'])
            if _inlend_idx >= _survinfo['ILNum']:
                _inlend_idx = _survinfo['ILNum'] - 1
            if _xlend_idx >= _survinfo['XLNum']:
                _xlend_idx = _survinfo['XLNum'] - 1
            if _zend_idx >= _survinfo['ZNum']:
                _zend_idx = _survinfo['ZNum'] - 1
            if _inlend_idx < _inlstart_idx:
                _inlend_idx = _inlstart_idx
            if _xlend_idx < _xlstart_idx:
                _xlend_idx = _xlstart_idx
            if _zend_idx < _zstart_idx:
                _zend_idx = _zstart_idx
            #
            _inlidx = np.arange(_inlstart_idx, _inlend_idx + 1,
                                self.cbbinlitvl.currentIndex() + 1, dtype=int)
            _xlidx = np.arange(_xlstart_idx, _xlend_idx + 1,
                               self.cbbxlitvl.currentIndex() + 1, dtype=int)
            _zidx = np.arange(_zstart_idx, _zend_idx + 1,
                              self.cbbzitvl.currentIndex() + 1, dtype=int)
            #
            _inl = _inlidx * _survinfo['ILStep'] + _survinfo['ILStart']
            _xl = _xlidx * _survinfo['XLStep'] + _survinfo['XLStart']
            _z = _zidx * _survinfo['ZStep'] + _survinfo['ZStart']
            #
            _pts = seis_ays.retrieveSeisILSliceFrom3DMat(np.zeros([self.survinfo['ZNum'], self.survinfo['XLNum'], self.survinfo['ILNum']]),
                                                         inlsls=_inl, verbose=False, seisinfo=self.survinfo)
            _pts = seis_ays.retrieveSeisXLSliceFrom3DMat(seis_ays.convertSeis2DMatTo3DMat(_pts), xlsls=_xl, verbose=False,
                                                         seisinfo=seis_ays.getSeisInfoFrom2DMat(_pts))
            _pts = seis_ays.retrieveSeisZSliceFrom3DMat(seis_ays.convertSeis2DMatTo3DMat(_pts), zsls=_z, verbose=False,
                                                        seisinfo=seis_ays.getSeisInfoFrom2DMat(_pts))
        #
        self.psseisdata[self.ldtsave.text()] = {}
        #
        _proplist = self.lwgattrib.selectedItems()
        _proplist = [f.text() for f in _proplist]
        #
        # Progress dialog
        _pgsdlg = QtWidgets.QProgressDialog()
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/seismic.png")),
                       QtGui.QIcon.Normal, QtGui.QIcon.Off)
        _pgsdlg.setWindowIcon(icon)
        _pgsdlg.setWindowTitle('Retrieve ' + str(len(_proplist)) + ' Property(s)')
        _pgsdlg.setCancelButton(None)
        _pgsdlg.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint)
        _pgsdlg.forceShow()
        _pgsdlg.setFixedWidth(400)
        _pgsdlg.setMaximum(len(_proplist))

        for _idx in range(len(_proplist)):
            QtCore.QCoreApplication.instance().processEvents()
            _pgsdlg.setValue(_idx)
            #
            self.psseisdata[self.ldtsave.text()][_proplist[_idx]] = {}
            #
            if self.rdbsrvall.isChecked():
                _data = np.reshape(self.seisdata[_proplist[_idx]],
                                     [self.survinfo['ILNum'], self.survinfo['XLNum'], self.survinfo['ZNum']])
                _info = self.survinfo
                _data = np.transpose(_data)
            if self.rdbsrvpart.isChecked():
                _data = self.seisdata[_proplist[_idx]]
                _data = seis_ays.retrieveSeisSampleFrom3DMat(_data, seisinfo=self.survinfo,
                                                             verbose=False,
                                                             samples=_pts)
                _info = seis_ays.getSeisInfoFrom2DMat(_data)
                _data = seis_ays.convertSeis2DMatTo3DMat(_data)
            #
            self.psseisdata[self.ldtsave.text()][_proplist[_idx]]['ShotData'] = _data
            self.psseisdata[self.ldtsave.text()][_proplist[_idx]]['ShotInfo'] = \
                psseis_ays.createShotInfoFromShotData(_data,
                                                      zstart=_info['ZStart'], zstep=_info['ZStep'],
                                                      xlstart=_info['XLStart'], xlstep=_info['XLStep'],
                                                      inlstart=_info['ILStart'], inlstep=_info['ILStep'])
        _pgsdlg.setValue(len(_proplist))
        #
        QtWidgets.QMessageBox.information(self.msgbox,
                                          "Convert Seismic to Pre-stack",
                                          str(len(_proplist)) + " properties converted successfully")

    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


    def checkSurvInfo(self):
        self.refreshMsgBox()
        #
        if seis_ays.checkSeisInfo(self.survinfo) is False:
            # print("ConvertSeis2PsSeis: Survey not found")
            # QtWidgets.QMessageBox.critical(self.msgbox,
            #                                'Convert Seismic to PointSet',
            #                                'Survey not found')
            return False
        return True

    def checkSeisData(self, f):
        self.refreshMsgBox()
        #
        return seis_ays.isSeis3DMatConsistentWithSeisInfo(self.seisdata[f], self.survinfo)


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    ConvertSeis2PsSeis = QtWidgets.QWidget()
    gui = convertseis2psseis()
    gui.setupGUI(ConvertSeis2PsSeis)
    ConvertSeis2PsSeis.show()
    sys.exit(app.exec_())


import os, sys
sys.path.append(os.path.dirname(__file__)[:-6][:-4][:-13])
from cognitivegeo.src.segpy.revisions import SEGY_REVISION_0, SEGY_REVISION_1

TEMPLATE = """
C 1 CLIENT { client             } COMPANY { company           } CREW NO {crew  }
C 2 LINE { line   } AREA { area               } MAP ID { map_id                }
C 3 REEL NO {reelnum} DAY-START OF REEL {d} YEAR {yr} OBSERVER {observer       }
C 4 INSTRUMENT: MFG { mfg    } MODEL { model  } SERIAL NO { serial             }
C 5 DATA TRACES/RECORD {dtpr} AUXILIARY TRACES/RECORD {atpr } CDP FOLD {cdpfold}
C 6 SAMPLE INTERVAL {intvl} SAMPLES/TRACE {spt} BITS/IN {bi} BYTES/SAMPLE {bps }
C 7 RECORDING FORMAT {rfmt} FORMAT THIS REEL {ftr } MEASUREMENT SYSTEM {measmnt}
C 8 SAMPLE CODE: FLOATING PT {f} FIXED PT {x} FIXED PT-GAIN {g} CORRELATED {cor}
C 9 GAIN  TYPE: FIXED {i} BINARY {b} FLOATING POINT {c} OTHER {other           }
C10 FILTERS: ALIAS {a }HZ  NOTCH {n }HZ  BAND {b1}-{b2 }HZ  SLOPE {s}-{s2}DB/OCT
C11 SOURCE: TYPE {type    } NUMBER/POINT {npt } POINT INTERVAL {point_interval }
C12     PATTERN: {source_pattern         } LENGTH {lent} WIDTH {width          }
C13 SWEEP: START {ss}HZ  END {se}HZ  LENGTH {lms}MS  CHANNEL NO {q} TYPE {cd   }
C14 TAPER: START LENGTH {tsl }MS  END LENGTH {tel }MS  TYPE {taper_type        }
C15 SPREAD: OFFSET {off } MAX DISTANCE {md  } GROUP INTERVAL {group_interval   }
C16 GEOPHONES: PER GROUP {p} SPACING {y} FREQUENCY {r} MFG {gmfg  } MODEL {gmod}
C17     PATTERN: {geophone_pattern      } LENGTH {glen} WIDTH {geophone_width  }
C18 TRACES SORTED BY: RECORD {u} CDP {v} OTHER { sort_other                    }
C19 AMPLITUDE RECOVERY: NONE {ar} SPHERICAL DIV {sd } AGC {} OTHER {ar_other   }
C20 MAP PROJECTION {map_projection  } ZONE ID {zid} COORDINATE UNITS {co_units }
C21 PROCESSING: { processing1                                                  }
C22 PROCESSING: { processing2                                                  }
C23 { unassigned1                                                              }
C24 { unassigned2                                                              }
C25 { unassigned3                                                              }
C26 { unassigned4                                                              }
C27 { unassigned5                                                              }
C28 { unassigned6                                                              }
C29 { unassigned7                                                              }
C30 { unassigned8                                                              }
C31 { unassigned9                                                              }
C32 { unassigned10                                                             }
C33 { unassigned11                                                             }
C34 { unassigned12                                                             }
C35 { unassigned13                                                             }
C36 { unassigned14                                                             }
C37 { unassigned15                                                             }
C38 { unassigned16                                                             }
C39 { unassigned17                                                             }
C40 { end_marker                                                               }
"""


END_TEXTUAL_HEADER = 'END TEXTUAL HEADER'
END_EBCDIC = 'END EBCDIC'

END_MARKERS = {SEGY_REVISION_0: END_EBCDIC,
               SEGY_REVISION_1: END_TEXTUAL_HEADER}

TEMPLATE_FIELD_NAMES = {'client': 'client',
                        'company': 'company',
                        'crew': 'crew_number',
                        'line': 'line',
                        'area': 'area',
                        'map_id': 'map_id',
                        'reelnum': 'reel_number',
                        'd': 'day_start_of_reel',
                        'yr': 'year',
                        'observer': 'observer',
                        'mfg': 'instrument_manufacturer',
                        'model': 'instrument_model',
                        'serial': 'instrument_serial',
                        'dtpr': 'data_traces_per_record',
                        'atpr': 'auxiliary_traces_per_record',
                        'cdpfold': 'cdp_fold',
                        'intvl': 'sample_interval',
                        'spt': 'samples_per_trace',
                        'bi': 'bits_per_inch',
                        'bps': 'bytes_per_sample',
                        'rfmt': 'recording_format',
                        'ftr': 'format_this_reel',
                        'measmnt': 'measurement_system',
                        'f': 'sample_code_floating_point',
                        'x': 'sample_code_fixed_point',
                        'g': 'sample_code_fixed_point_gain',
                        'cor': 'sample_code_correlated',
                        'i': 'gain_type_fixed',
                        'b': 'gain_type_binary',
                        'c': 'gain_type_fixed_point_gain',
                        'other': 'gain_type_other',
                        'a': 'filters_alias_hz',
                        'n': 'filters_notch_hz',
                        'b1': 'filters_band_lower_hz',
                        'b2': 'filters_band_upper_hz',
                        's': 'filters_slope_lower_db_per_oct',
                        's2': 'filters_slope_upper_db_per_oct',
                        'type': 'source_type',
                        'npt': 'source_number_per_point',
                        'point_interval': 'source_point_interval',
                        'source_pattern': 'source_pattern',
                        'lent': 'source_length',
                        'width': 'source_width',
                        'ss': 'sweep_start_hz',
                        'se': 'sweep_end_hz',
                        'lms': 'sweep_length_ms',
                        'q': 'sweep_channel_number',
                        'cd': 'sweep_type',
                        'tsl': 'taper_start_length_ms',
                        'tel': 'taper_end_length_ms',
                        'taper_type': 'taper_type',
                        'off': 'spread_offset',
                        'md': 'spread_max_distance',
                        'group_interval': 'spread_group_interval',
                        'p': 'geophones_per_group',
                        'y': 'geophone_spacing',
                        'r': 'geophone_frequency',
                        'gmfg': 'geophone_manufacturer',
                        'gmod': 'geophone_model',
                        'geophone_pattern': 'geophone_pattern',
                        'glen': 'geophone_length',
                        'geophone_width': 'geophone_width',
                        'u': 'traces_sorted_by_record',
                        'v': 'traces_sorted_by_cdp',
                        'sort_other': 'traces_sorted_by_other',
                        'ar': 'amplitude_recovery_none',
                        'sd': 'amplitude_recovery_spherical_div',
                        '': 'amplitude_recovery_agc',
                        'ar_other': 'amplitude_recovery_other',
                        'map_projection': 'map_projection',
                        'zid': 'zone_id',
                        'co_units': 'coordinate_units',
                        'processing1': 'processing1',
                        'processing2': 'processing2',
                        'unassigned1': 'unassigned1',
                        'unassigned2': 'unassigned2',
                        'unassigned3': 'unassigned3',
                        'unassigned4': 'unassigned4',
                        'unassigned5': 'unassigned5',
                        'unassigned6': 'unassigned6',
                        'unassigned7': 'unassigned7',
                        'unassigned8': 'unassigned8',
                        'unassigned9': 'unassigned9',
                        'unassigned10': 'unassigned10',
                        'unassigned11': 'unassigned11',
                        'unassigned12': 'unassigned12',
                        'unassigned13': 'unassigned13',
                        'unassigned14': 'unassigned14',
                        'unassigned15': 'unassigned15',
                        'unassigned16': 'unassigned16',
                        'unassigned17': 'unassigned17',
                        'end_marker':   'end_marker'
                        }

INV_TEMPLATE_FIELD_NAMES = dict((v, k) for k, v in TEMPLATE_FIELD_NAMES.items())



#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# Create a window for view information about GeoPy


from PyQt5 import QtCore, QtGui, QtWidgets
import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)

class about(object):

    rootpath = ''
    #
    iconpath = os.path.dirname(__file__)

    def setupGUI(self, About):
        About.setObjectName("About")
        About.setFixedSize(430, 230)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/about.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        About.setWindowIcon(icon)
        self.lblicon = QtWidgets.QLabel(About)
        self.lblicon.setObjectName("lblicon")
        self.lblicon.setGeometry(QtCore.QRect(30, 30, 100, 100))
        self.lbltitle = QtWidgets.QLabel(About)
        self.lbltitle.setObjectName("lbltitle")
        self.lbltitle.setGeometry(QtCore.QRect(150, 30, 320, 40))
        self.lbldate = QtWidgets.QLabel(About)
        self.lbldate.setObjectName("lbldate")
        self.lbldate.setGeometry(QtCore.QRect(150, 70, 320, 60))
        self.lblcopyright = QtWidgets.QLabel(About)
        self.lblcopyright.setObjectName("lblcopyright")
        self.lblcopyright.setGeometry(QtCore.QRect(150, 130, 320, 60))
        self.lbllink = QtWidgets.QLabel(About)
        self.lbllink.setObjectName("lbllink")
        self.lbllink.setGeometry(QtCore.QRect(275, 131, 70, 30))
        self.lblqr = QtWidgets.QLabel(About)
        self.lblqr.setObjectName("lblqr")
        self.lblqr.setGeometry(QtCore.QRect(340, 140, 80, 80))
        #
        self.retranslateGUI(About)
        QtCore.QMetaObject.connectSlotsByName(About)


    def retranslateGUI(self, About):
        _translate = QtCore.QCoreApplication.translate
        About.setWindowTitle(_translate("About", "About Cognitive Geo"))
        self.lblicon.setPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/logo.png")).scaled(80, 80, QtCore.Qt.KeepAspectRatio))
        self.lblicon.setAlignment(QtCore.Qt.AlignCenter)
        self.lbltitle.setText(_translate("About", "Cognitive Geo"))
        self.lbltitle.setFont(QtGui.QFont("Helvetica", 16, QtGui.QFont.Bold))
        self.lbltitle.setAlignment(QtCore.Qt.AlignTop)
        self.lbldate.setText(_translate("About", "Version 1.1\nBuilt on PyQt5, Vispy, Numpy, Matlibplot, Scipy"))
        self.lblcopyright.setText(_translate("About", "Copyright (C) 2018-2020          \n\n\tMore information? Scan me"))
        self.lbllink.setText(_translate("About", '<a href="https://www.linkedin.com/in/geopy-team-182b27185/">GeoPy Team</a>'))
        self.lbllink.setOpenExternalLinks(True)
        self.lblqr.setPixmap(
            QtGui.QPixmap(os.path.join(self.iconpath, "icons/qrcode.png")).scaled(80, 80, QtCore.Qt.KeepAspectRatio))
        self.lblqr.setAlignment(QtCore.Qt.AlignCenter)


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    About = QtWidgets.QWidget()
    gui = about()
    gui.setupGUI(About)
    About.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# basic functions for processing curves

import numpy as np
from scipy import interpolate
import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.vis.messager import messager as vis_msg


__all__ = ['curve']


def changeCurveSize(curve, length_new, kind='cubic'):
    """
    Change curve size through 1D interpolation (from scipy)

    Args:
        curve:      2D array of curves, with each row representing one curve
        length_new: Length of the new curve.
        kind:       Interpolation type as a string ('linear', 'nearest', 'quadratic', 'cubic'). Default is 'cubic'

    Return:
        2D array of curves after interpolation, with each row representing one curve
    """

    if np.ndim(curve) != 2:
        vis_msg.print("ERROR in changeCurveSize: 2D array expected", type='error')
        sys.exit()
    if length_new <= 1:
        vis_msg.print('ERROR in changeCurveSize: New curve length > 1', type='error')
        sys.exit()

    ncurve = np.shape(curve)[0]
    length = np.shape(curve)[1]

    line = np.linspace(0.0, 1.0, length)
    line_new = np.linspace(0.0, 1.0, length_new)

    curve_new = np.zeros([ncurve, length_new])
    for i in range(ncurve):
        curve_i = curve[i, :]
        f = interpolate.interp1d(line, curve_i, kind=kind)
        curve_new[i, :] = f(line_new)

    return curve_new


class curve:
    # Pack all functions as a class
    #
    changeCurveSize = changeCurveSize


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

# Create a GUI for settings (GUI)

from PyQt5 import QtCore, QtGui, QtWidgets
import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.core.settings import settings as core_set

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class settingsgui(object):

    mainwindow = None
    settings = core_set.GUI
    #
    iconpath = os.path.dirname(__file__)
    dialog = None


    def setupGUI(self, SettingsGUI):
        SettingsGUI.setObjectName("SettingsGUI")
        SettingsGUI.setFixedSize(400, 130)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/logo.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        SettingsGUI.setWindowIcon(icon)
        #
        self.lbltoolbar = QtWidgets.QLabel(SettingsGUI)
        self.lbltoolbar.setObjectName("lbltoolbar")
        self.lbltoolbar.setGeometry(QtCore.QRect(10, 10, 50, 30))
        self.cbxtoolbarleft = QtWidgets.QCheckBox(SettingsGUI)
        self.cbxtoolbarleft.setObjectName("cbxtoolbarleft")
        self.cbxtoolbarleft.setGeometry(QtCore.QRect(30, 40, 170, 30))
        self.cbxtoolbarright = QtWidgets.QCheckBox(SettingsGUI)
        self.cbxtoolbarright.setObjectName("cbxtoolbarright")
        self.cbxtoolbarright.setGeometry(QtCore.QRect(230, 40, 170, 30))
        self.cbxtoolbartop = QtWidgets.QCheckBox(SettingsGUI)
        self.cbxtoolbartop.setObjectName("cbxtoolbartop")
        self.cbxtoolbartop.setGeometry(QtCore.QRect(30, 80, 170, 30))
        self.cbxtoolbarbottom = QtWidgets.QCheckBox(SettingsGUI)
        self.cbxtoolbarbottom.setObjectName("cbxtoolbartop")
        self.cbxtoolbarbottom.setGeometry(QtCore.QRect(230, 80, 170, 30))
        #
        self.msgbox = QtWidgets.QMessageBox(SettingsGUI)
        self.msgbox.setObjectName("msgbox")
        _center_x = SettingsGUI.geometry().center().x()
        _center_y = SettingsGUI.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(SettingsGUI)
        QtCore.QMetaObject.connectSlotsByName(SettingsGUI)


    def retranslateGUI(self, SettingsGUI):
        self.dialog = SettingsGUI
        #
        _translate = QtCore.QCoreApplication.translate
        SettingsGUI.setWindowTitle(_translate("SettingsGUI", "Interface Settings"))
        self.lbltoolbar.setText(_translate("SettingsGUI", "Toolbars:"))
        self.cbxtoolbarleft.setText(_translate("SettingsGUI", "Data import && export"))
        self.cbxtoolbarleft.stateChanged.connect(self.changeCbxToolbarLeft)
        self.cbxtoolbarright.setText(_translate("SettingsGUI", "Featured toolbox"))
        self.cbxtoolbarright.stateChanged.connect(self.changeCbxToolbarRight)
        self.cbxtoolbartop.setText(_translate("SettingsGUI", "Manage survey && data"))
        self.cbxtoolbartop.stateChanged.connect(self.changeCbxToolbarTop)
        self.cbxtoolbarbottom.setText(_translate("SettingsGUI", "Visualization"))
        self.cbxtoolbarbottom.stateChanged.connect(self.changeCbxToolbarBottom)
        if self.mainwindow is not None:
            self.cbxtoolbarleft.setChecked(self.settings['Toolbar']['Left'])
            self.cbxtoolbarright.setChecked(self.settings['Toolbar']['Right'])
            self.cbxtoolbartop.setChecked(self.settings['Toolbar']['Top'])
            self.cbxtoolbarbottom.setChecked(self.settings['Toolbar']['Bottom'])
            self.mainwindow.toolbarleft.setVisible(self.settings['Toolbar']['Left'])
            self.mainwindow.toolbarright.setVisible(self.settings['Toolbar']['Right'])
            self.mainwindow.toolbartop.setVisible(self.settings['Toolbar']['Top'])
            self.mainwindow.toolbarbottom.setVisible(self.settings['Toolbar']['Bottom'])


    def changeCbxToolbarLeft(self):
        self.refreshMsgBox()
        #
        if self.mainwindow is None:
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'GeoPy',
                                           'No GeoPy found')
            return
        if self.cbxtoolbarleft.isChecked():
            self.mainwindow.toolbarleft.setVisible(True)
        else:
            self.mainwindow.toolbarleft.setVisible(False)
        self.settings['Toolbar']['Left'] = self.cbxtoolbarleft.isChecked()


    def changeCbxToolbarRight(self):
        self.refreshMsgBox()
        #
        if self.mainwindow is None:
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'GeoPy',
                                           'No GeoPy found')
            return
        if self.cbxtoolbarright.isChecked():
            self.mainwindow.toolbarright.setVisible(True)
        else:
            self.mainwindow.toolbarright.setVisible(False)
        self.settings['Toolbar']['Right'] = self.cbxtoolbarright.isChecked()


    def changeCbxToolbarTop(self):
        self.refreshMsgBox()
        #
        if self.mainwindow is None:
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'GeoPy',
                                           'No GeoPy found')
            return
        if self.cbxtoolbartop.isChecked():
            self.mainwindow.toolbartop.setVisible(True)
        else:
            self.mainwindow.toolbartop.setVisible(False)
        self.settings['Toolbar']['Top'] = self.cbxtoolbartop.isChecked()


    def changeCbxToolbarBottom(self):
        self.refreshMsgBox()
        #
        if self.mainwindow is None:
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           'GeoPy',
                                           'No GeoPy found')
            return
        if self.cbxtoolbarbottom.isChecked():
            self.mainwindow.toolbarbottom.setVisible(True)
        else:
            self.mainwindow.toolbarbottom.setVisible(False)
        self.settings['Toolbar']['Bottom'] = self.cbxtoolbarbottom.isChecked()


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    SettingsGUI = QtWidgets.QWidget()
    gui = settingsgui()
    gui.setupGUI(SettingsGUI)
    SettingsGUI.show()
    sys.exit(app.exec_())


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     March 2018                                                                      #
#                                                                                           #
#############################################################################################

from __future__ import absolute_import, division, print_function

__version__ = '1.1'

__all__ = ['basic', 'core', 'vis', 'seismic', 'psseismic', 'pointset', 'gui',
           'start']

import os, sys
#
sys.path.append(os.path.dirname(__file__)[:-13])

import cognitivegeo.src.basic as basic
import cognitivegeo.src.core as core
import cognitivegeo.src.vis as vis
import cognitivegeo.src.seismic as seismic
import cognitivegeo.src.psseismic as psseismic
import cognitivegeo.src.pointset as pointset
import cognitivegeo.src.gui as gui

def start(path=os.path.dirname(__file__)):
    gui.start(startpath=path)

if __name__ == "__main__":
    start()


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     February 2019                                                                   #
#                                                                                           #
#############################################################################################

# basic functions for fonts

import matplotlib.pyplot as plt
import sys, os
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.vis.color import color as color


__all__ = ['font']

# List of all available font properties, including Name, Style, Weight, and Size
FontNameList = ['Arial', 'Helvetica', 'Segoe UI', 'Tahoma', 'Times New Roman', 'Verdana']
FontStyleList = ['Normal', 'Italic', 'Oblique']
FontWeightList = ['Normal', 'Light', 'Medium', 'Bold', 'Semibold', 'Heavy', 'Black']
FontSizeList = [i for i in range(1, 50)]

def updatePltFont(fontstyle):
    """
    Update the font style in matplotlib

    Args:
        fontstyle: Font style dictionary with the following keys: Name, Size, Weight, Color,

    Return:
        N/A
    """
    if fontstyle is None or len(fontstyle.keys()) < 1:
        return
    #
    if 'Name' in fontstyle.keys():
        plt.rcParams['font.sans-serif'] = fontstyle['Name']
    if 'Size' in fontstyle.keys():
        plt.rcParams['font.size'] = fontstyle['Size']
        plt.rcParams['axes.titlesize'] = fontstyle['Size']
        plt.rcParams['axes.labelsize'] = fontstyle['Size']
    if 'Weight' in fontstyle.keys():
        plt.rcParams['font.weight'] = fontstyle['Weight'].lower()
        plt.rcParams['axes.titleweight'] = fontstyle['Weight'].lower()
        plt.rcParams['axes.labelweight'] = fontstyle['Weight'].lower()
    if 'Color' in fontstyle.keys():
        plt.rcParams['text.color'] = fontstyle['Color'].lower()
        plt.rcParams['axes.labelcolor'] = fontstyle['Color'].lower()
        plt.rcParams['xtick.color'] = fontstyle['Color'].lower()
        plt.rcParams['ytick.color'] = fontstyle['Color'].lower()


class font:
    # Pack all functions as a class
    #
    FontNameList = FontNameList
    FontColorList = color.ColorList
    FontStyleList = FontStyleList
    FontWeightList = FontWeightList
    FontSizeList = FontSizeList
    #
    updatePltFont = updatePltFont



import time
import os
import sys

from itertools import (islice, cycle, tee, chain, repeat)

NATIVE_ENDIANNESS = '<' if sys.byteorder == 'little' else '>'

EMPTY_BYTE_STRING = b''

UNSET = object()

def pairwise(iterable):
    """Pairwise iteration.

    Args:
        iterable: An iterable series.

    Returns:
        An iterator over 2-tuples.

    Raises:
        StopIteration: If the iterable contains fewer than two items.
    """
    a, b = tee(iterable)
    next(b)
    yield from zip(a, b)


def batched(iterable, batch_size, padding=UNSET):
    """Batch an iterable series into equal sized batches.

    Args:
        iterable: The series to be batched.

        batch_size: The size of the batch. Must be at least one.

        padding: Optional value used to pad the final batch to batch_size. If
            omitted, the final batch may be smaller than batch_size.

    Yields:
        A series of lists, each containing batch_size items from iterable.

    Raises:
        ValueError: If batch_size is less than one.
    """
    if batch_size < 1:
        raise ValueError("Batch size {} is not at least one.".format(batch_size))

    pending = []

    for item in iterable:
        pending.append(item)
        if len(pending) == batch_size:
            batch = pending
            pending = []
            yield batch

    num_left_over = len(pending)
    if num_left_over > 0:
        if padding is not UNSET:
            pending.extend([padding] * (batch_size - num_left_over))
        yield pending


def pad(iterable, padding=None, size=None):
    if size is None:
        return chain(iterable, repeat(padding))
    return islice(pad(iterable, padding), size)


def complementary_intervals(intervals, start=None, stop=None):
    """Compute a complementary set of intervals which alternate with given intervals to form a contiguous range.

    Given,

        Start                          Stop
           [-----)    [-----) [----)

    produces,

        [--)     [----)     [-)    [---)

    Args:
        intervals: An sequence of at least one existing slices or ranges. The type of the first interval (slice or
            range) is used as the result type.
        start: An optional start index, defaults to the start of the first slice.
        stop: An optional one-beyond-the-end index, defaults to the stop attribute of the last slice.

    Yields:
        A complementary series of slices which alternate with the supplied slices.  The number of returned
        slices will always be len(slices) + 1 since both leading and trailing slices will always be returned.
        Note the some of the returned slices may be 'empty' (having zero length).
    """
    if len(intervals) < 1:
        raise ValueError("intervals must contain at least one interval (slice or range) object")

    interval_type = type(intervals[0])

    if start is None:
        start = intervals[0].start

    if stop is None:
        stop = intervals[-1].stop

    index = start
    for s in intervals:
        yield interval_type(index, s.start)
        index = s.stop

    yield interval_type(index, stop)

def intervals_are_contiguous(intervals):
    """Determine whether a series of intervals are contiguous.

    Args:
        intervals: An iterable series of intervals where each interval is either
            a range or slice object.

    Returns:
        True if the intervals are in order, contiguous and non-overlapping,
        otherwise False.
    """

    for a, b in pairwise(intervals):
        if a.stop != b.start:
            return False
    return True


def intervals_partially_overlap(interval_a, interval_b):
    """Determine whether two intervals partially overlap.

    Args:
        interval_a: A range or slice object.
        interval_b: A range or slice object.

    Returns:
        True if interval_a partially overlaps interval_b, otherwise False if the intervals
        are either disjoint or exactly coincident.
    """
    if interval_a == interval_b:
        return False

    if interval_a.start <= interval_b.start:
        first_interval = interval_a
        second_interval = interval_b
    else:
        first_interval = interval_b
        second_interval = interval_a

    return second_interval.start < first_interval.stop


def roundrobin(*iterables):
    """Take items from each iterable in turn until all iterables are exhausted.

    roundrobin('ABC', 'D', 'EF') --> A D E B F C
    """
    # Recipe credited to George Sakkis
    pending = len(iterables)
    nexts = cycle(iter(it).__next__ for it in iterables)
    while pending:
        try:
            for n in nexts:
                yield n()
        except StopIteration:
            pending -= 1
            nexts = cycle(islice(nexts, pending))


def contains_duplicates(sorted_iterable):
    """Determine in an iterable series contains duplicates.

    Args:
        sorted_iterable: Any iterable series which must be sorted in either
            ascending or descending order.

    Returns:
        True if sorted_iterable contains duplicates, otherwise False.
    """
    for a, b in pairwise(sorted_iterable):
        if a == b:
            return True
    return False


def measure_stride(iterable):
    """Determine whether successive numeric items differ by a constant amount.

    Args:
        iterable: An iterable series of numeric values.

    Returns:
        The difference between successive values (e.g. item[1] - item[0]) if
        that difference is the same between all successive pairs, otherwise
        None.
    """
    stride = None
    for a, b in pairwise(iterable):
        new_stride = b - a
        if stride is None:
            stride = new_stride
        elif stride != new_stride:
            return None
    return stride


def minmax(iterable):
    """Return the minimum and maximum of an iterable series.

    This function requires only a single pass over the data.

    Args:
        iterable: An iterable series for which to determine the minimum and
            maximum values.

    Returns:
        A 2-tuple containing the minimum and maximum values.
    """
    iterator = iter(iterable)
    try:
        first = next(iterator)
    except StopIteration:
        raise ValueError("minmax() arg is an empty iterable series")
    minimum = first
    maximum = first
    for item in iterator:
        minimum = min(minimum, item)
        maximum = max(maximum, item)
    return minimum, maximum


def file_length(fh):
    """Determine the length of a file-like object in bytes.

    Args:
        fh: A seekable file-like-object.

    Returns:
        An integer length in bytes.
    """
    pos = fh.tell()
    try:
        fh.seek(0, os.SEEK_END)
        return fh.tell()
    finally:
        fh.seek(pos, os.SEEK_SET)


def filename_from_handle(fh):
    """Determine the name of the file underlying a file-like object.

    Args:
        fh: A file-like object.

    Returns:
        A string containing the file name, or '<unknown>' if it could not
        be determined.
    """
    try:
        return fh.name
    except AttributeError:
        return '<unknown>'


def now_millis():
    millis = int(round(time.time() * 1000))
    return millis


def round_up(integer, multiple):
    """Round up to the nearest multiple"""
    return integer if integer % multiple == 0 else integer + multiple - integer % multiple


def underscores_to_camelcase(s):
    """Convert text_in_this_style to TextInThisStyle."""
    return ''.join(w.capitalize() for w in s.split('_'))


def first_sentence(s):
    sentence, stop, _ = s.partition('.')
    return sentence + stop


def lower_first(s):
    """Lower case the first character of a string."""
    return s[:1].lower() + s[1:]


def almost_equal(x, y, epsilon):
    max_xy_one = max(1.0, abs(x), abs(y))
    e = epsilon * max_xy_one
    delta = abs(x - y)
    return delta <= e


def ensure_contains(collection, item):
    return collection if item in collection else conjoin(collection, item)


def conjoin(collection, item):
    return collection + type(collection)((item,))


def is_magic_name(name):
    return len(name) > 4 and name.startswith('__') and name.endswith('__')


def super_class(cls):
    """Return the next class in the MRO of cls."""
    mro = cls.mro()
    assert len(mro) > 0
    if len(mro) == 1:
        assert mro[0] is object
        return object
    return mro[1]


def flatten(sequence_of_sequences):
    return chain.from_iterable(sequence_of_sequences)


def four_bytes(byte_str):
    a, b, c, d = byte_str[:4]
    return a, b, c, d


#############################################################################################
#                                                                                           #
# Author:   GeoPy Team                                                                      #
# Email:    geopy.info@gmail.com                                                            #
# Date:     January 2019                                                                    #
#                                                                                           #
#############################################################################################

# Create a window for pointset crossplot


from PyQt5 import QtCore, QtGui, QtWidgets
import os, sys
import numpy as np
#
sys.path.append(os.path.dirname(__file__)[:-4][:-4][:-13])
from cognitivegeo.src.basic.data import data as basic_data
from cognitivegeo.src.basic.matdict import matdict as basic_mdt
from cognitivegeo.src.core.settings import settings as core_set
from cognitivegeo.src.pointset.visualization import visualization as point_vis
from cognitivegeo.src.gui.configlineplotting import configlineplotting as gui_configlineplotting
from cognitivegeo.src.vis.messager import messager as vis_msg

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)
QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps, True)


class plotvis2dpointsetcrossplt(object):

    pointsetdata = {}
    linestyle = core_set.Visual['Line']
    fontstyle = core_set.Visual['Font']
    #
    iconpath = os.path.dirname(__file__)
    dialog = None
    #
    featurelist = []
    lineplottingconfig = {}

    def setupGUI(self, PlotVis2DPointSetCrossplt):
        PlotVis2DPointSetCrossplt.setObjectName("PlotVis2DPointSetCrossplt")
        PlotVis2DPointSetCrossplt.setFixedSize(420, 390)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/plotpoint.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        PlotVis2DPointSetCrossplt.setWindowIcon(icon)
        #
        self.lblpoint = QtWidgets.QLabel(PlotVis2DPointSetCrossplt)
        self.lblpoint.setObjectName("lblpoint")
        self.lblpoint.setGeometry(QtCore.QRect(10, 10, 150, 30))
        self.twgpoint = QtWidgets.QTableWidget(PlotVis2DPointSetCrossplt)
        self.twgpoint.setObjectName("twgpoint")
        self.twgpoint.setGeometry(QtCore.QRect(10, 50, 240, 270))
        self.twgpoint.setColumnCount(2)
        self.twgpoint.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.twgpoint.verticalHeader().hide()
        #
        self.btnconfigline = QtWidgets.QPushButton(PlotVis2DPointSetCrossplt)
        self.btnconfigline.setObjectName("btnconfigline")
        self.btnconfigline.setGeometry(QtCore.QRect(380, 10, 30, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/settings.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnconfigline.setIcon(icon)
        #
        self.lblxaxis = QtWidgets.QLabel(PlotVis2DPointSetCrossplt)
        self.lblxaxis.setObjectName("lblxaxis")
        self.lblxaxis.setGeometry(QtCore.QRect(270, 50, 40, 30))
        self.cbbxfeature = QtWidgets.QComboBox(PlotVis2DPointSetCrossplt)
        self.cbbxfeature.setObjectName("cbbxfeature")
        self.cbbxfeature.setGeometry(QtCore.QRect(270, 80, 140, 30))
        self.ldtxmin = QtWidgets.QLineEdit(PlotVis2DPointSetCrossplt)
        self.ldtxmin.setObjectName("ldtxmin")
        self.ldtxmin.setGeometry(QtCore.QRect(270, 120, 60, 30))
        self.ldtxmin.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtxmax = QtWidgets.QLineEdit(PlotVis2DPointSetCrossplt)
        self.ldtxmax.setObjectName("ldtxmax")
        self.ldtxmax.setGeometry(QtCore.QRect(350, 120, 60, 30))
        self.ldtxmax.setAlignment(QtCore.Qt.AlignCenter)
        self.lblxrangeto = QtWidgets.QLabel(PlotVis2DPointSetCrossplt)
        self.lblxrangeto.setObjectName("lblxrangeto")
        self.lblxrangeto.setGeometry(QtCore.QRect(330, 120, 20, 30))
        self.lblxrangeto.setAlignment(QtCore.Qt.AlignCenter)
        self.lblyaxis = QtWidgets.QLabel(PlotVis2DPointSetCrossplt)
        self.lblyaxis.setObjectName("lblyaxis")
        self.lblyaxis.setGeometry(QtCore.QRect(270, 170, 40, 30))
        self.cbbyfeature = QtWidgets.QComboBox(PlotVis2DPointSetCrossplt)
        self.cbbyfeature.setObjectName("cbbyfeature")
        self.cbbyfeature.setGeometry(QtCore.QRect(270, 200, 140, 30))
        self.ldtymin = QtWidgets.QLineEdit(PlotVis2DPointSetCrossplt)
        self.ldtymin.setObjectName("ldtymin")
        self.ldtymin.setGeometry(QtCore.QRect(270, 240, 60, 30))
        self.ldtymin.setAlignment(QtCore.Qt.AlignCenter)
        self.ldtymax = QtWidgets.QLineEdit(PlotVis2DPointSetCrossplt)
        self.ldtymax.setObjectName("ldtymax")
        self.ldtymax.setGeometry(QtCore.QRect(350, 240, 60, 30))
        self.ldtymax.setAlignment(QtCore.Qt.AlignCenter)
        self.lblyrangeto = QtWidgets.QLabel(PlotVis2DPointSetCrossplt)
        self.lblyrangeto.setObjectName("lblyrangeto")
        self.lblyrangeto.setGeometry(QtCore.QRect(330, 240, 20, 30))
        self.lblyrangeto.setAlignment(QtCore.Qt.AlignCenter)
        self.lbllegend = QtWidgets.QLabel(PlotVis2DPointSetCrossplt)
        self.lbllegend.setObjectName("lbllegend")
        self.lbllegend.setGeometry(QtCore.QRect(270, 290, 60, 30))
        self.cbblegend = QtWidgets.QComboBox(PlotVis2DPointSetCrossplt)
        self.cbblegend.setObjectName("cbblegend")
        self.cbblegend.setGeometry(QtCore.QRect(330, 290, 80, 30))
        #
        self.btnplot = QtWidgets.QPushButton(PlotVis2DPointSetCrossplt)
        self.btnplot.setObjectName("btnplot")
        self.btnplot.setGeometry(QtCore.QRect(160, 340, 100, 30))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(os.path.join(self.iconpath, "icons/plotpoint.png")),
                       QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.btnplot.setIcon(icon)
        #
        self.msgbox = QtWidgets.QMessageBox(PlotVis2DPointSetCrossplt)
        self.msgbox.setObjectName("msgbox")
        _center_x = PlotVis2DPointSetCrossplt.geometry().center().x()
        _center_y = PlotVis2DPointSetCrossplt.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))
        #
        self.retranslateGUI(PlotVis2DPointSetCrossplt)
        QtCore.QMetaObject.connectSlotsByName(PlotVis2DPointSetCrossplt)


    def retranslateGUI(self, PlotVis2DPointSetCrossplt):
        self.dialog = PlotVis2DPointSetCrossplt
        #
        _translate = QtCore.QCoreApplication.translate
        PlotVis2DPointSetCrossplt.setWindowTitle(_translate("PlotVis2DPointSetCrossplt", "2D Window: PointSet Cross-plot"))
        self.lblpoint.setText(_translate("PlotVis2DPointSetCrossplt", "Select pointsets:"))
        #
        self.twgpoint.setHorizontalHeaderLabels(["Name", "Length"])
        if len(self.pointsetdata.keys()) > 0:
            _idx = 0
            self.twgpoint.setRowCount(len(self.pointsetdata.keys()))
            for i in sorted(self.pointsetdata.keys()):
                item = QtWidgets.QTableWidgetItem()
                item.setText(i)
                item.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
                self.twgpoint.setItem(_idx, 0, item)
                #
                item = QtWidgets.QTableWidgetItem()
                item.setText(str(basic_mdt.maxDictConstantRow(self.pointsetdata[i])))
                item.setTextAlignment(QtCore.Qt.AlignCenter)
                item.setFlags(QtCore.Qt.ItemIsEditable)
                self.twgpoint.setItem(_idx, 1, item)
                #
                _idx = _idx + 1
            self.twgpoint.setRowCount(_idx)
        self.twgpoint.itemSelectionChanged.connect(self.changeTwgPoint)
        #
        self.btnconfigline.setText(_translate("PlotVis2DPointSetCrossplt", ""))
        self.btnconfigline.clicked.connect(self.clickBtnConfigLine)
        for _item in self.twgpoint.selectedItems():
            _idx = _item.row()
            _name = self.twgpoint.item(_idx, 0).text()
            _config = self.linestyle
            self.lineplottingconfig[_name] = _config
        #
        self.lblxaxis.setText(_translate("PlotVis2DPointSetCrossplt", "X-axis:"))
        self.cbbxfeature.currentIndexChanged.connect(self.changeCbbXFeature)
        self.lblxrangeto.setText(_translate("PlotVis2DPointSetCrossplt", "~~"))
        self.lblyaxis.setText(_translate("PlotVis2DPointSetCrossplt", "Y-axis:"))
        self.lblyrangeto.setText(_translate("PlotVis2DPointSetCrossplt", "~~"))
        self.cbbyfeature.currentIndexChanged.connect(self.changeCbbYFeature)
        #
        self.lbllegend.setText(_translate("PlotVis2DPointSetCrossplt", "Legend:"))
        self.cbblegend.addItems(['On', 'Off'])
        #
        self.btnplot.setText(_translate("PlotVis2DPointSetCrossplt", "Cross-Plot"))
        self.btnplot.setDefault(True)
        self.btnplot.clicked.connect(self.clickBtnPlot)


    def changeTwgPoint(self):
        self.cbbxfeature.clear()
        self.cbbyfeature.clear()
        #
        _featurelist = []
        if len(self.twgpoint.selectedItems()) > 0:
            _featurelist = self.twgpoint.selectedItems()[0].row()
            _featurelist = self.twgpoint.item(_featurelist, 0).text()
            _featurelist = self.pointsetdata[_featurelist].keys()
        #
        for _item in self.twgpoint.selectedItems():
            _idx = _item.row()
            _name = self.twgpoint.item(_idx, 0).text()
            _featurelist = list(set(_featurelist) & set(self.pointsetdata[_name].keys()))
        #
        self.featurelist = _featurelist
        self.cbbxfeature.addItems(self.featurelist)
        self.cbbyfeature.addItems(self.featurelist)
        #
        _config = {}
        for _item in self.twgpoint.selectedItems():
            _idx = _item.row()
            _name = self.twgpoint.item(_idx, 0).text()
            if _name in self.lineplottingconfig.keys():
                _config[_name] = self.lineplottingconfig[_name]
            else:
                _config[_name] = self.linestyle
        self.lineplottingconfig = _config


    def clickBtnConfigLine(self):
        _config = QtWidgets.QDialog()
        _gui = gui_configlineplotting()
        _gui.lineplottingconfig = self.lineplottingconfig
        _gui.setupGUI(_config)
        _config.exec()
        self.lineplottingconfig = _gui.lineplottingconfig
        _config.show()


    def changeCbbXFeature(self):
        if self.cbbxfeature.currentIndex() < 0:
            self.ldtxmin.setText('')
            self.ldtxmax.setText('')
        else:
            _min = 1e+9
            _max = -1e+9
            _f = self.featurelist[self.cbbxfeature.currentIndex()]
            for _point in self.twgpoint.selectedItems():
                _idx = _point.row()
                _name = self.twgpoint.item(_idx, 0).text()
                if _min > np.min(self.pointsetdata[_name][_f]):
                    _min = np.min(self.pointsetdata[_name][_f])
                if _max < np.max(self.pointsetdata[_name][_f]):
                    _max = np.max(self.pointsetdata[_name][_f])
            self.ldtxmin.setText(str(_min))
            self.ldtxmax.setText(str(_max))


    def changeCbbYFeature(self):
        if self.cbbyfeature.currentIndex() < 0:
            self.ldtymin.setText('')
            self.ldtymax.setText('')
        else:
            _min = 1e+9
            _max = -1e+9
            _f = self.featurelist[self.cbbyfeature.currentIndex()]
            for _point in self.twgpoint.selectedItems():
                _idx = _point.row()
                _name = self.twgpoint.item(_idx, 0).text()
                if _min > np.min(self.pointsetdata[_name][_f]):
                    _min = np.min(self.pointsetdata[_name][_f])
                if _max < np.max(self.pointsetdata[_name][_f]):
                    _max = np.max(self.pointsetdata[_name][_f])
            self.ldtymin.setText(str(_min))
            self.ldtymax.setText(str(_max))


    def clickBtnPlot(self):
        self.refreshMsgBox()
        #
        _npoint = len(self.twgpoint.selectedItems())
        if _npoint < 1:
            vis_msg.print("ERROR in PlotVis2DPointSetCrossplt: No pointset selected", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           '2D Window: PointSet Cross-plot',
                                           'No pointset selected')
            return
        #
        _xfeature = self.featurelist[self.cbbxfeature.currentIndex()]
        _yfeature = self.featurelist[self.cbbyfeature.currentIndex()]
        _xmin = basic_data.str2float(self.ldtxmin.text())
        _xmax = basic_data.str2float(self.ldtxmax.text())
        _ymin = basic_data.str2float(self.ldtymin.text())
        _ymax = basic_data.str2float(self.ldtymax.text())
        if _xmin is False or _xmax is False or _ymin is False or _ymax is False:
            vis_msg.print("ERROR in PlotVis2DPointSetCrossplt: Non-float range specified for plot", type='error')
            QtWidgets.QMessageBox.critical(self.msgbox,
                                           '2D Window: PointSet Cross-plot',
                                           'on-float range specified for plot')
            return
        #
        _legendon = False
        if self.cbblegend.currentIndex() == 0:
            _legendon = True
        #
        _pointdict = {}
        _markerstylelist = []
        _markersizelist = []
        _colorlist = []
        _linestylelist = []
        _linewidthlist = []
        for _item in self.twgpoint.selectedItems():
            _idx = _item.row()
            _name = self.twgpoint.item(_idx, 0).text()

            #
            if _xfeature not in self.pointsetdata[_name].keys():
                vis_msg.print("ERROR in PlotVis2DPointSetCrossplt: X-feature not found in " + _name, type='error')
                QtWidgets.QMessageBox.critical(self.msgbox,
                                               '2-D Cross-plot',
                                               'X-feature not found in ' + _name)
                return
            if _yfeature not in self.pointsetdata[_name].keys():
                vis_msg.print("ERROR in PlotVis2DPointSetCrossplt: Y-feature not found in " + _name, type='error')
                QtWidgets.QMessageBox.critical(self.msgbox,
                                               '2-D Cross-plot',
                                               'Y-feature not found in ' + _name)
                return
            #
            _data = {}
            _data[_xfeature] = self.pointsetdata[_name][_xfeature]
            _data[_yfeature] = self.pointsetdata[_name][_yfeature]
            _pointdict[_name] = _data
            #
            _markerstylelist.append(self.lineplottingconfig[_name]['MarkerStyle'])
            _markersizelist.append(self.lineplottingconfig[_name]['MarkerSize'])
            _colorlist.append(self.lineplottingconfig[_name]['Color'].lower())
            _linestylelist.append(self.lineplottingconfig[_name]['Style'].lower())
            _linewidthlist.append(self.lineplottingconfig[_name]['Width'])
        #
        point_vis.crossplot2D(_pointdict,
                              colorlist=_colorlist, linestylelist=_linestylelist,
                              linewidthlist=_linewidthlist,
                              markerstylelist=_markerstylelist,
                              markersizelist=_markersizelist,
                              xfeature=_xfeature, yfeature=_yfeature,
                              xlim=[_xmin, _xmax], ylim=[_ymin, _ymax],
                              xlabel=_xfeature, ylabel=_yfeature, legendon=_legendon,
                              fontstyle=self.fontstyle,
                              qicon=QtGui.QIcon(os.path.join(self.iconpath, "icons/logo.png")))


    def refreshMsgBox(self):
        _center_x = self.dialog.geometry().center().x()
        _center_y = self.dialog.geometry().center().y()
        self.msgbox.setGeometry(QtCore.QRect(_center_x - 150, _center_y - 50, 300, 100))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    PlotVis2DPointSetCrossplt = QtWidgets.QWidget()
    gui = plotvis2dpointsetcrossplt()
    gui.setupGUI(PlotVis2DPointSetCrossplt)
    PlotVis2DPointSetCrossplt.show()
    sys.exit(app.exec_())